/*
 * acast.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <string>
#include <exception/AlibException.h>
#include <factory/XmlDataFactory.hpp>
#include <sax/SaxParseInterface.h>
#include <sax/ParserException.h>
#include <object/Object.h>
#include "CastApi.hpp"

// -----------------------------------------------------------------------------------------

int main(int argc, char** argv) {
	try {
		TCLAP::CmdLine cmd("cast binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to echo",	false,	"-",		"file");
		cmd.add( input );

		TCLAP::ValueArg<std::string> type(	"t",	"type",		"Output type",		true,	"",		"string");
		cmd.add( type );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		std::chrono::measurements::start("Overal", std::chrono::measurements::Type::OVERALL);
		std::chrono::measurements::start("Input read", std::chrono::measurements::Type::AUXILARY);
		std::deque<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		alib::Object in = alib::XmlDataFactory::fromTokens<alib::Object>(tokens);

		std::chrono::measurements::end();
		std::chrono::measurements::start("Cast", std::chrono::measurements::Type::MAIN);

		alib::Object res = alib::castApi::getCastPool(type.getValue()).cast(in.getData());

		std::chrono::measurements::end();
		std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

		alib::XmlDataFactory::toStdout(res);

		std::chrono::measurements::end();
		std::chrono::measurements::end();

		if(measure.getValue()) std::clog << std::chrono::measurements::results() << std::endl;

		return 0;
	} catch(const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
