/*
 * aconvert.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Martin Zak
 */

#include <tclap/CmdLine.h>
#include <fstream>

#include "automaton/Automaton.h"
#include "factory/XmlDataFactory.hpp"
#include "factory/StringDataFactory.hpp"
#include "exception/AlibException.h"

#include "DotConverter.h"
#include "GasTexConverter.h"

void automatonFromString(std::istream& in, std::ostream& out) {
	automaton::Automaton automaton = alib::StringDataFactory::fromStream<automaton::Automaton>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	alib::XmlDataFactory::toStream( automaton, out);
}

void grammarFromString(std::istream& in, std::ostream& out) {
	grammar::Grammar grammar = alib::StringDataFactory::fromStream<grammar::Grammar>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	alib::XmlDataFactory::toStream( grammar, out);
}

void regexpFromString(std::istream& in, std::ostream& out) {
	regexp::RegExp regexp = alib::StringDataFactory::fromStream<regexp::RegExp>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	alib::XmlDataFactory::toStream( regexp, out );
}

void stringFromString(std::istream& in, std::ostream& out) {
	string::String string = alib::StringDataFactory::fromStream<string::String>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	alib::XmlDataFactory::toStream(string, out);
}

void automatonToString(std::istream& in, std::ostream& out) {
	automaton::Automaton automaton = alib::XmlDataFactory::fromStream<automaton::Automaton>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	alib::StringDataFactory::toStream( automaton, out);
}

void grammarToString(std::istream& in, std::ostream& out) {
	grammar::Grammar grammar = alib::XmlDataFactory::fromStream<grammar::Grammar>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	alib::StringDataFactory::toStream( grammar, out);
}

void regexpToString(std::istream& in, std::ostream& out) {
	regexp::RegExp regexp = alib::XmlDataFactory::fromStream<regexp::RegExp>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	alib::StringDataFactory::toStream( regexp, out);
}

void stringToString(std::istream& in, std::ostream& out) {
	string::String string = alib::XmlDataFactory::fromStream<string::String>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	alib::StringDataFactory::toStream( string, out);
}

void automatonToDot(std::istream& in, std::ostream& out) {
	automaton::Automaton automaton = alib::XmlDataFactory::fromStream<automaton::Automaton>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	DotConverter::convert(out, automaton);
}

void automatonToGasTex(std::istream& in, std::ostream& out) {
	automaton::Automaton automaton = alib::XmlDataFactory::fromStream<automaton::Automaton>(in);

	std::chrono::measurements::end();
	std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

	GasTexConverter::convert(out, automaton);
}

int main(int argc, char* argv[]) {
	try {
		TCLAP::CmdLine cmd("Convert binary", ' ', "0.01");

		TCLAP::SwitchArg automaton_from_string(	"", "automaton_from_string",	"Convert automaton from string representation");
		cmd.add( automaton_from_string );

		TCLAP::SwitchArg automaton_to_string(	"", "automaton_to_string",	"Convert automaton to string representation");
		cmd.add( automaton_to_string );

		TCLAP::SwitchArg automaton_to_dot(	"", "automaton_to_dot",		"Convert automaton to dot representation");
		cmd.add( automaton_to_dot );

		TCLAP::SwitchArg automaton_to_gastex(	"", "automaton_to_gastex",	"Convert automaton to gastex representation");
		cmd.add( automaton_to_gastex );

		TCLAP::SwitchArg grammar_from_string(	"", "grammar_from_string",	"Convert grammar from string representation");
		cmd.add( grammar_from_string );

		TCLAP::SwitchArg grammar_to_string(	"", "grammar_to_string",	"Convert grammar to string representation");
		cmd.add( grammar_to_string );

		TCLAP::SwitchArg regexp_from_string(	"", "regexp_from_string",	"Convert regexp from string representation");
		cmd.add( regexp_from_string );

		TCLAP::SwitchArg regexp_to_string(	"", "regexp_to_string",		"Convert regexp to string representation");
		cmd.add( regexp_to_string );

		TCLAP::SwitchArg string_from_string(	"", "string_from_string",	"Convert string from string representation");
		cmd.add( string_from_string );

		TCLAP::SwitchArg string_to_string(	"","string_to_string",		"Convert string to string representation");
		cmd.add( string_to_string );

		TCLAP::ValueArg<std::string> input(	"i", "input", "Input file", false, "-", "file");
		cmd.add( input );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		std::chrono::measurements::start("Overal", std::chrono::measurements::Type::OVERALL);
		std::chrono::measurements::start("Input read", std::chrono::measurements::Type::AUXILARY);

		std::istream* in = NULL;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				in = &std::cin;
			} else {
				in = new std::ifstream(input.getValue());
			}
		} else {
			in = &std::cin;
		}

		if(automaton_from_string.getValue()) {
			automatonFromString(*in, std::cout);
		} else if(automaton_to_string.getValue()) {
			automatonToString(*in, std::cout);
		} else if(automaton_to_dot.getValue()) {
			automatonToDot(*in, std::cout);
		} else if(automaton_to_gastex.getValue()) {
			automatonToGasTex(*in, std::cout);
		} else if(grammar_from_string.getValue()) {
			grammarFromString(*in, std::cout);
		} else if(grammar_to_string.getValue()) {
			grammarToString(*in, std::cout);
		} else if(regexp_from_string.getValue()) {
			regexpFromString(*in, std::cout);
		} else if(regexp_to_string.getValue()) {
			regexpToString(*in, std::cout);
		} else if(string_from_string.getValue()) {
			stringFromString(*in, std::cout);
		} else if(string_to_string.getValue()) {
			stringToString(*in, std::cout);
		}

		if(input.isSet() && input.getValue() != "-") {
			delete in;
		}

		std::chrono::measurements::end();
		std::chrono::measurements::end();

		if(measure.getValue()) std::clog << std::chrono::measurements::results() << std::endl;

		return 0;
	} catch (const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}

