/*
 * aecho.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Martin Zak
 */

#include <tclap/CmdLine.h>
#include <string>
#include <exception/AlibException.h>
#include <factory/XmlDataFactory.hpp>
#include <sax/SaxParseInterface.h>
#include <sax/ParserException.h>
#include <object/Object.h>

#include <chrono>

int main(int argc, char** argv) {
	try {
		TCLAP::CmdLine cmd("echo binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to echo",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		std::chrono::measurements::start("Overal", std::chrono::measurements::Type::OVERALL);
		std::chrono::measurements::start("Input read", std::chrono::measurements::Type::AUXILARY);

		std::deque<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		alib::Object object = alib::XmlDataFactory::fromTokens<alib::Object>(tokens);

		std::chrono::measurements::end();
		std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

		alib::XmlDataFactory::toStdout(object);

		std::chrono::measurements::end();
		std::chrono::measurements::end();

		if(measure.getValue()) std::clog << std::chrono::measurements::results() << std::endl;

		return 0;
	} catch(const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
