/*
 * Determinize.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#include "Determinize.h"
#include "../transform/PDAToRHPDA.h"
#include "../transform/RHPDAToPDA.h"
#include "automaton/PDA/NPDA.h"
#include "automaton/PDA/RealTimeHeightDeterministicNPDA.h"

namespace automaton {

namespace determinize {

automaton::Automaton Determinize::determinize(const automaton::Automaton& automaton) {
	return getInstance().dispatch(automaton.getData());
}

DFA Determinize::determinize(const automaton::DFA& automaton) {
	return automaton;
}

auto DeterminizeDFA = Determinize::RegistratorWrapper<automaton::DFA, automaton::DFA>(Determinize::getInstance(), Determinize::determinize);

DPDA Determinize::determinize(const automaton::DPDA& automaton) {
	return automaton;
}

auto DeterminizeDPDA = Determinize::RegistratorWrapper<automaton::DPDA, automaton::DPDA>(Determinize::getInstance(), Determinize::determinize);

SinglePopDPDA Determinize::determinize(const automaton::SinglePopDPDA& automaton) {
	return automaton;
}

auto DeterminizeSinglePopDPDA = Determinize::RegistratorWrapper<automaton::SinglePopDPDA, automaton::SinglePopDPDA>(Determinize::getInstance(), Determinize::determinize);

InputDrivenDPDA Determinize::determinize(const automaton::InputDrivenDPDA& automaton) {
	return automaton;
}

auto DeterminizeInputDrivenDPDA = Determinize::RegistratorWrapper<automaton::InputDrivenDPDA, automaton::InputDrivenDPDA>(Determinize::getInstance(), Determinize::determinize);

VisiblyPushdownDPDA Determinize::determinize(const automaton::VisiblyPushdownDPDA& automaton) {
	return automaton;
}

auto DeterminizeVisiblyPushdownDPDA = Determinize::RegistratorWrapper<automaton::VisiblyPushdownDPDA, automaton::VisiblyPushdownDPDA>(Determinize::getInstance(), Determinize::determinize);

RealTimeHeightDeterministicDPDA Determinize::determinize(const automaton::RealTimeHeightDeterministicDPDA& automaton) {
	return automaton;
}

auto DeterminizeRealTimeHeightDeterministicDPDA = Determinize::RegistratorWrapper<automaton::RealTimeHeightDeterministicDPDA, automaton::RealTimeHeightDeterministicDPDA>(Determinize::getInstance(), Determinize::determinize);

DPDA Determinize::determinize(const automaton::NPDA& automaton) {
	automaton::RealTimeHeightDeterministicNPDA rhpda = automaton::PDAToRHPDA::convert(automaton);
	automaton::RealTimeHeightDeterministicDPDA dpda = Determinize::determinize(rhpda);
	return automaton::RHPDAToPDA::convert(dpda);
}

auto DeterminizeNPDA = Determinize::RegistratorWrapper<automaton::DPDA, automaton::NPDA>(Determinize::getInstance(), Determinize::determinize);

OneTapeDTM Determinize::determinize(const automaton::OneTapeDTM& automaton) {
	return automaton;
}

auto DeterminizeOneTapeDTM = Determinize::RegistratorWrapper<automaton::OneTapeDTM, automaton::OneTapeDTM>(Determinize::getInstance(), Determinize::determinize);

DFTA Determinize::determinize(const automaton::DFTA& automaton) {
	return automaton;
}

auto DeterminizeDFTA = Determinize::RegistratorWrapper<automaton::DFTA, automaton::DFTA>(Determinize::getInstance(), Determinize::determinize);

} /* namespace determinize */

} /* namespace automaton */

#include "DeterminizeNFAPart.cxx"
#include "DeterminizeIDPDAPart.cxx"
#include "DeterminizeVPAPart.cxx"
#include "DeterminizeRHDPDAPart.cxx"
#include "DeterminizeNFTAPart.cxx"

