/*
 * Occurrences.h
 *
 *  Created on: 9. 2. 2014
 *	  Author: Jan Travnicek
 */

#ifndef _AUTOMATON_OCCURRENCES_H__
#define _AUTOMATON_OCCURRENCES_H__

#include <common/multipleDispatch.hpp>
#include <automaton/Automaton.h>
#include <object/Object.h>
#include <string/LinearString.h>
#include <tree/ranked/RankedTree.h>

namespace automaton {

namespace run {

class Occurrences : public std::DoubleDispatch<std::set<unsigned>, automaton::AutomatonBase, alib::ObjectBase> {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static std::set<unsigned> occurrences(const automaton::Automaton& automaton, const alib::Object& object);
	static std::set<unsigned> occurrences(const automaton::Automaton& automaton, const string::LinearString& string);
	static std::set<unsigned> occurrences(const automaton::Automaton& automaton, const tree::RankedTree& string);

	static std::set<unsigned> occurrences(const automaton::DFA& automaton, const string::LinearString& string);
	static std::set<unsigned> occurrences(const automaton::DPDA& automaton, const string::LinearString& string);
	static std::set<unsigned> occurrences(const automaton::DFTA& automaton, const tree::RankedTree & tree);
private:
	static State occurrences(const automaton::DFTA & automaton, const tree::RankedNode & node, std::set<unsigned> & occ, unsigned & i);

public:
	static Occurrences& getInstance() {
		static Occurrences res;
		return res;
	}
};

} /* namespace run */

} /* namespace automaton */

#endif /* _AUTOMATON_OCCURRENCES_H__ */
