/*
 * EpsilonRemoverIncoming.cpp
 *
 *  Created on: 24. 11. 2014
 *	  Author: Jan Travnicek
 */

#include "EpsilonRemoverIncoming.h"

#include "../properties/EpsilonClosure.h"

namespace automaton {

namespace simplify {

automaton::DFA EpsilonRemoverIncoming::remove(const automaton::DFA& origFSM) {
	return origFSM;
}

auto EpsilonRemoverIncomingDFA = EpsilonRemoverIncoming::RegistratorWrapper<automaton::DFA, automaton::DFA>(EpsilonRemoverIncoming::getInstance(), EpsilonRemoverIncoming::remove);

automaton::MultiInitialStateNFA EpsilonRemoverIncoming::remove(const automaton::MultiInitialStateNFA& origFSM) {
	return origFSM;
}

auto EpsilonRemoverIncomingMultiInitialStateNFA = EpsilonRemoverIncoming::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::MultiInitialStateNFA>(EpsilonRemoverIncoming::getInstance(), EpsilonRemoverIncoming::remove);

automaton::NFA EpsilonRemoverIncoming::remove(const automaton::NFA& origFSM) {
	return origFSM;
}

auto EpsilonRemoverIncomingNFA = EpsilonRemoverIncoming::RegistratorWrapper<automaton::NFA, automaton::NFA>(EpsilonRemoverIncoming::getInstance(), EpsilonRemoverIncoming::remove);

automaton::NFA EpsilonRemoverIncoming::remove( const automaton::EpsilonNFA & origFSM ) {
	automaton::NFA fsm(origFSM.getInitialState());

	fsm.setStates( origFSM.getStates() );
	fsm.setInputAlphabet( origFSM.getInputAlphabet() );

	std::map<std::pair<automaton::State, alphabet::Symbol>, std::set<automaton::State>> origTransitions = origFSM.getSymbolTransitions();

	/**
	 * Step 1 from Melichar 2.41
	 */
	for( const auto & from : origFSM.getStates( ) )
		for( const auto & fromClosure : automaton::properties::EpsilonClosure::epsilonClosure( origFSM, from ) )
			for( const auto & symbol : origFSM.getInputAlphabet() ) {
				auto transitions = origTransitions.find(std::make_pair(fromClosure, symbol));
				if(transitions == origTransitions.end()) continue;

				for( const auto & to : transitions->second )
					fsm.addTransition( from, symbol, to );
			}

	/**
	 * Step 2 from Melichar 2.41
	 */
	const std::set<automaton::State> & F = origFSM.getFinalStates( );
	for( const auto & q : fsm.getStates( ) )
	{
		const std::set<automaton::State> & cl = automaton::properties::EpsilonClosure::epsilonClosure( origFSM, q );
		std::set<automaton::State> intersect;

		set_intersection( cl.begin(), cl.end(), F.begin(), F.end(), std::inserter( intersect, intersect.begin() ) );
		if( intersect.size( ) != 0 )
			fsm.addFinalState( q );
	}

	return fsm;
}

auto EpsilonRemoverIncomingEpsilonNFA = EpsilonRemoverIncoming::RegistratorWrapper<automaton::NFA, automaton::EpsilonNFA>(EpsilonRemoverIncoming::getInstance(), EpsilonRemoverIncoming::remove);

automaton::Automaton EpsilonRemoverIncoming::remove(const automaton::Automaton& automaton) {
	return getInstance().dispatch(automaton.getData());
}

} /* namespace simplify */

} /* namespace automaton */
