/*
 * ToAutomatonBototmUp.cpp
 *
 * Created on: 20. 5. 2015
 * Author: Tomas Pecka
 */
#include "ToAutomatonBottomUp.h"

#include <alphabet/BottomOfTheStackSymbol.h>
#include <alphabet/LabeledSymbol.h>
#include <exception/AlibException.h>

namespace grammar {

namespace convert {

automaton::Automaton ToAutomatonBottomUp::convert(const grammar::Grammar& grammar) {
	 return getInstance().dispatch(grammar.getData());
}

template <class T>
automaton::NPDA ToAutomatonBottomUp::convert(const T& grammar) {
	automaton::State q = automaton::State('q');
	automaton::State r = automaton::State('r');

	automaton::NPDA automaton(q, alphabet::Symbol(alphabet::BottomOfTheStackSymbol::BOTTOM_OF_THE_STACK));
	automaton.addState(r);
	automaton.addFinalState(r);

	automaton.setInputAlphabet(grammar.getTerminalAlphabet());

	for(const alphabet::Symbol& symbol : grammar.getNonterminalAlphabet())
		automaton.addStackSymbol(symbol);
	for(const alphabet::Symbol& symbol : grammar.getTerminalAlphabet())
		automaton.addStackSymbol(symbol);

	for(const alphabet::Symbol& symbol : grammar.getTerminalAlphabet())
		automaton.addTransition(automaton.getInitialState(), symbol, std::vector<alphabet::Symbol>{}, automaton.getInitialState(), std::vector<alphabet::Symbol>{symbol});

	//for(const std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>>& kv : grammar.getRules())
	for(const auto& kv : grammar.getRules())
		for(const std::vector<alphabet::Symbol>& rhs : kv.second)
			automaton.addTransition(automaton.getInitialState(), rhs, automaton.getInitialState(), std::vector<alphabet::Symbol>{kv.first});

	automaton.addTransition(automaton.getInitialState(), std::vector<alphabet::Symbol>{alphabet::Symbol(alphabet::BottomOfTheStackSymbol::BOTTOM_OF_THE_STACK), grammar.getInitialSymbol()}, r, std::vector<alphabet::Symbol>{});

	return automaton;
}

auto ToAutomatonBottomUpCFG = ToAutomatonBottomUp::RegistratorWrapper<automaton::NPDA, grammar::CFG>(ToAutomatonBottomUp::getInstance(), ToAutomatonBottomUp::convert);
auto ToAutomatonBottomUpEpsilonFreeCFG = ToAutomatonBottomUp::RegistratorWrapper<automaton::NPDA, grammar::EpsilonFreeCFG>(ToAutomatonBottomUp::getInstance(), ToAutomatonBottomUp::convert);

} /* namespace convert */

} /* namespace grammar */
