/*
 * ToGrammarLeftRG.cpp
 *
 *  Created on: 8. 3. 2014
 *      Author: Tomas Pecka
 */

#include "ToGrammarLeftRG.h"

#include <exception/AlibException.h>

namespace grammar {

namespace convert {

grammar::LeftRG ToGrammarLeftRG::convert(const grammar::Grammar& grammar) {
	return getInstance().dispatch(grammar.getData());
}

grammar::LeftRG ToGrammarLeftRG::convert(const grammar::RightRG& grammar) {
	// 1.
	alphabet::Symbol s = alphabet::createUniqueSymbol( grammar.getInitialSymbol( ), grammar.getNonterminalAlphabet(), grammar.getTerminalAlphabet() );

	grammar::LeftRG lrg(s);

	for(const auto & nonterminalSymbol : grammar.getNonterminalAlphabet()) {
		lrg.addNonterminalSymbol( nonterminalSymbol );
	}

	lrg.setTerminalAlphabet( grammar.getTerminalAlphabet( ) );
	lrg.setGeneratesEpsilon( grammar.getGeneratesEpsilon( ) );

	// 2.
	for( const auto & rule : grammar.getRules( ) ) {
		const alphabet::Symbol& lhs = rule.first;

		for(const auto & ruleRHS : rule.second ) {
		        if( ruleRHS.is<std::pair<alphabet::Symbol, alphabet::Symbol>>( ) ) {
				const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();

				alphabet::Symbol leftSide = rhs.second;
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair( lhs, rhs.first );
				lrg.addRule( leftSide, rightSide );

				if( lhs == grammar.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = rhs.second;
					alphabet::Symbol rightSide = rhs.first;
					lrg.addRule( leftSide, rightSide );
				}
			} else {
				const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();

				alphabet::Symbol leftSide = lrg.getInitialSymbol( );
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair ( lhs, rhs );
				lrg.addRule( leftSide, rightSide );

				if( lhs == grammar.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = lrg.getInitialSymbol( );
					alphabet::Symbol rightSide = rhs;
					lrg.addRule( leftSide, rightSide );
				}
			}
		}
	}
	return lrg;
}

auto ToGrammarLeftRGRightRG = ToGrammarLeftRG::RegistratorWrapper<grammar::LeftRG, grammar::RightRG>(ToGrammarLeftRG::getInstance(), ToGrammarLeftRG::convert);

} /* namespace convert */

} /* namespace grammar */
