/*
 * NullableNonterminals.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef NULLABLE_NONTERMINALS_H_
#define NULLABLE_NONTERMINALS_H_

#include <common/multipleDispatch.hpp>

#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class NullableNonterminals : public std::SingleDispatch<std::set<alphabet::Symbol>, grammar::GrammarBase> {
public:
	static std::set<alphabet::Symbol> getNullableNonterminals( const grammar::Grammar & grammar );

	/**
	 * Retrieve all nullable nonterminals from grammar
	 * Nullable nonterminal is such nonterminal A for which holds that A ->^* \eps
	 *
	 * Source: Melichar, algorithm 2.4, step 1
	 *
	 * @param grammar grammar
	 * @return set of nullable nonterminals from grammar
	 */
	template<class T>
	static std::set<alphabet::Symbol> getNullableNonterminals(const T& grammar);

	static NullableNonterminals& getInstance() {
		static NullableNonterminals res;
		return res;
	}
};

} /* namespace properties */

} /* namespace grammar */

#endif /* NULLABLE_NONTERMINALS_H_ */
