/*
 * EpsilonRemover.cpp
 *
 *  Created on: 24. 11. 2014
 *	  Author: Jan Travnicek
 */

#include "EpsilonRemover.h"

#include "../properties/NullableNonterminals.h"

namespace grammar {

namespace simplify {

void removeNullableNonterminals(grammar::EpsilonFreeCFG& grammar, const std::set<alphabet::Symbol>& nullableNonterminals, const alphabet::Symbol& lhs, const std::vector<alphabet::Symbol>& rhs, unsigned i, std::vector<alphabet::Symbol> clear) {
	if(rhs.size() == i) {
		if(clear.size() == 0) return;
		grammar.addRule(lhs, clear);
	} else if(nullableNonterminals.find(rhs[i]) != nullableNonterminals.end()) {
		removeNullableNonterminals(grammar, nullableNonterminals, lhs, rhs, i+1, clear);
		clear.push_back(rhs[i]);
		removeNullableNonterminals(grammar, nullableNonterminals, lhs, rhs, i+1, clear);
	} else {
		clear.push_back(rhs[i]);
		removeNullableNonterminals(grammar, nullableNonterminals, lhs, rhs, i+1, clear);
	}
}

template<class T>
grammar::EpsilonFreeCFG removeInternal( const T & origGrammar ) {
	grammar::EpsilonFreeCFG grammar(origGrammar.getInitialSymbol());

	for( const auto & symbol : origGrammar.getNonterminalAlphabet() )
		grammar.addNonterminalSymbol( symbol );

	for( const auto & symbol : origGrammar.getTerminalAlphabet() )
		grammar.addTerminalSymbol( symbol );

	std::set<alphabet::Symbol> nullableNonterminals = grammar::properties::NullableNonterminals::getNullableNonterminals(origGrammar);

	for( const auto & rule : origGrammar.getRawRules( ) )
		for(const auto & rhs : rule.second) {
			if(rhs.size() == 0) continue;

			removeNullableNonterminals(grammar, nullableNonterminals, rule.first, rhs, 0, {});
		}

	if(nullableNonterminals.find(origGrammar.getInitialSymbol()) != nullableNonterminals.end())
		grammar.setGeneratesEpsilon(true);

	return grammar;
}

grammar::EpsilonFreeCFG EpsilonRemover::remove(const grammar::CFG& origGrammar) {
	return removeInternal(origGrammar);
}

auto EpsilonRemoverCFG = EpsilonRemover::RegistratorWrapper<grammar::EpsilonFreeCFG, grammar::CFG>(EpsilonRemover::getInstance(), EpsilonRemover::remove);

grammar::EpsilonFreeCFG EpsilonRemover::remove(const grammar::EpsilonFreeCFG& origGrammar) {
	return origGrammar;
}

auto EpsilonRemoverEpsilonFreeCFG = EpsilonRemover::RegistratorWrapper<grammar::EpsilonFreeCFG, grammar::EpsilonFreeCFG>(EpsilonRemover::getInstance(), EpsilonRemover::remove);

grammar::CNF EpsilonRemover::remove(const grammar::CNF& origGrammar) {
	return origGrammar;
}

auto EpsilonRemoverCNF = EpsilonRemover::RegistratorWrapper<grammar::CNF, grammar::CNF>(EpsilonRemover::getInstance(), EpsilonRemover::remove);

grammar::GNF EpsilonRemover::remove(const grammar::GNF& origGrammar) {
	return origGrammar;
}

auto EpsilonRemoverGNF = EpsilonRemover::RegistratorWrapper<grammar::GNF, grammar::GNF>(EpsilonRemover::getInstance(), EpsilonRemover::remove);

grammar::EpsilonFreeCFG EpsilonRemover::remove(const grammar::LG& origGrammar) {
	return removeInternal(origGrammar);
}

auto EpsilonRemoverLG = EpsilonRemover::RegistratorWrapper<grammar::EpsilonFreeCFG, grammar::LG>(EpsilonRemover::getInstance(), EpsilonRemover::remove);

grammar::EpsilonFreeCFG EpsilonRemover::remove(const grammar::LeftLG& origGrammar) {
	return removeInternal(origGrammar);
}

auto EpsilonRemoverLeftLG = EpsilonRemover::RegistratorWrapper<grammar::EpsilonFreeCFG, grammar::LeftLG>(EpsilonRemover::getInstance(), EpsilonRemover::remove);

grammar::LeftRG EpsilonRemover::remove(const grammar::LeftRG& origGrammar) {
	return origGrammar;
}

auto EpsilonRemoverLeftRG = EpsilonRemover::RegistratorWrapper<grammar::LeftRG, grammar::LeftRG>(EpsilonRemover::getInstance(), EpsilonRemover::remove);

grammar::EpsilonFreeCFG EpsilonRemover::remove(const grammar::RightLG& origGrammar) {
	return removeInternal(origGrammar);
}

auto EpsilonRemoverRightLG = EpsilonRemover::RegistratorWrapper<grammar::EpsilonFreeCFG, grammar::RightLG>(EpsilonRemover::getInstance(), EpsilonRemover::remove);

grammar::RightRG EpsilonRemover::remove(const grammar::RightRG& origGrammar) {
	return origGrammar;
}

auto EpsilonRemoverRightRG = EpsilonRemover::RegistratorWrapper<grammar::RightRG, grammar::RightRG>(EpsilonRemover::getInstance(), EpsilonRemover::remove);

grammar::Grammar EpsilonRemover::remove(const grammar::Grammar& grammar) {
	return getInstance().dispatch(grammar.getData());
}

} /* namespace simplify */

} /* namespace grammar */
