/*
 * SimpleRulesRemover.h
 *
 *  Created on: 24. 11. 2014
 *	  Author: Jan Travnicek
 */

#ifndef SIMPLE_RULES_REMOVER_H_
#define SIMPLE_RULES_REMOVER_H_

#include <common/multipleDispatch.hpp>
#include <map>
#include <algorithm>

#include <grammar/Grammar.h>

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>
#include <exception/AlibException.h>

namespace grammar {

namespace simplify {

class SimpleRulesRemover : public std::SingleDispatch<grammar::Grammar, grammar::GrammarBase> {
public:
	static grammar::Grammar remove( const grammar::Grammar & grammar );

	static grammar::CFG remove( const grammar::CFG & grammar );
	static grammar::EpsilonFreeCFG remove( const grammar::EpsilonFreeCFG & grammar );
	static grammar::GNF remove( const grammar::GNF & grammar );
	static grammar::CNF remove( const grammar::CNF & grammar );
	static grammar::LG remove( const grammar::LG & grammar );
	static grammar::LeftLG remove( const grammar::LeftLG & grammar );
	static grammar::LeftRG remove( const grammar::LeftRG & grammar );
	static grammar::RightLG remove( const grammar::RightLG & grammar );
	static grammar::RightRG remove( const grammar::RightRG & grammar );

	static SimpleRulesRemover& getInstance() {
		static SimpleRulesRemover res;
		return res;
	}
};

} /* namespace simplify */

} /* namespace grammar */

#endif /* SIMPLE_RULES_REMOVER_H_ */
