/*
 * Trim.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "Trim.h"

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include "UnreachableSymbolsRemover.h"
#include "UnproductiveSymbolsRemover.h"

namespace grammar {

namespace simplify {

template<class T>
T Trim::trim( const T & grammar ) {
	return grammar::simplify::UnreachableSymbolsRemover::remove( grammar::simplify::UnproductiveSymbolsRemover::remove( grammar ) );
}

auto TrimCFG = Trim::RegistratorWrapper<grammar::CFG, grammar::CFG>(Trim::getInstance(), Trim::trim);
auto TrimEpsilonFreeCFG = Trim::RegistratorWrapper<grammar::EpsilonFreeCFG, grammar::EpsilonFreeCFG>(Trim::getInstance(), Trim::trim);
auto TrimGNF = Trim::RegistratorWrapper<grammar::GNF, grammar::GNF>(Trim::getInstance(), Trim::trim);
auto TrimCNF = Trim::RegistratorWrapper<grammar::CNF, grammar::CNF>(Trim::getInstance(), Trim::trim);
auto TrimLG = Trim::RegistratorWrapper<grammar::LG, grammar::LG>(Trim::getInstance(), Trim::trim);
auto TrimLeftLG = Trim::RegistratorWrapper<grammar::LeftLG, grammar::LeftLG>(Trim::getInstance(), Trim::trim);
auto TrimLeftRG = Trim::RegistratorWrapper<grammar::LeftRG, grammar::LeftRG>(Trim::getInstance(), Trim::trim);
auto TrimRightLG = Trim::RegistratorWrapper<grammar::RightLG, grammar::RightLG>(Trim::getInstance(), Trim::trim);
auto TrimRightRG = Trim::RegistratorWrapper<grammar::RightRG, grammar::RightRG>(Trim::getInstance(), Trim::trim);

grammar::Grammar Trim::trim(const grammar::Grammar& grammar) {
	return getInstance().dispatch(grammar.getData());
}

} /* namespace simplify */

} /* namespace grammar */

