/*
 * UnproductiveSymbolsRemover.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "UnproductiveSymbolsRemover.h"

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include <set>
#include <algorithm>

#include "../properties/ProductiveNonterminals.h"

namespace grammar {

namespace simplify {

template<class T>
T UnproductiveSymbolsRemover::remove( const T & grammar ) {
	// 1.
	std::set<alphabet::Symbol> Nt = grammar::properties::ProductiveNonterminals::getProductiveNonterminals( grammar );

	T ret(grammar.getInitialSymbol( ) );

	for( const auto & symbol : Nt )
		ret.addNonterminalSymbol( symbol );

	for( const auto & symbol : grammar.getTerminalAlphabet( ) )
		ret.addTerminalSymbol( symbol );

	const std::set<alphabet::Symbol> & terminals = ret.getTerminalAlphabet( );
	for( const auto & rule : grammar.getRawRules( ) ) {
		if( Nt.count( rule.first ) ) {
			for( const auto & rhs : rule.second ) {
				if( all_of( rhs.begin( ), rhs.end( ), [ Nt, terminals ]( const alphabet::Symbol & symbol ) {
						return Nt.count( symbol ) || terminals.count( symbol );
				} ) )
					ret.addRawRule( rule.first, rhs );
			}
		}
	}


	/* if( ! G1.getNonTerminalSymbols( ) . count( grammar.getInitialSymbol( ) ) )
		throw AlibException( "Starting symbol of grammar was marked as unproductive and therefore it was removed." ); */

	// 2.
	return ret;
}

auto UnproductiveSymbolsRemoverCFG = UnproductiveSymbolsRemover::RegistratorWrapper<grammar::CFG, grammar::CFG>(UnproductiveSymbolsRemover::getInstance(), UnproductiveSymbolsRemover::remove);
auto UnproductiveSymbolsRemoverEpsilonFreeCFG = UnproductiveSymbolsRemover::RegistratorWrapper<grammar::EpsilonFreeCFG, grammar::EpsilonFreeCFG>(UnproductiveSymbolsRemover::getInstance(), UnproductiveSymbolsRemover::remove);
auto UnproductiveSymbolsRemoverGNF = UnproductiveSymbolsRemover::RegistratorWrapper<grammar::GNF, grammar::GNF>(UnproductiveSymbolsRemover::getInstance(), UnproductiveSymbolsRemover::remove);
auto UnproductiveSymbolsRemoverCNF = UnproductiveSymbolsRemover::RegistratorWrapper<grammar::CNF, grammar::CNF>(UnproductiveSymbolsRemover::getInstance(), UnproductiveSymbolsRemover::remove);
auto UnproductiveSymbolsRemoverLG = UnproductiveSymbolsRemover::RegistratorWrapper<grammar::LG, grammar::LG>(UnproductiveSymbolsRemover::getInstance(), UnproductiveSymbolsRemover::remove);
auto UnproductiveSymbolsRemoverLeftLG = UnproductiveSymbolsRemover::RegistratorWrapper<grammar::LeftLG, grammar::LeftLG>(UnproductiveSymbolsRemover::getInstance(), UnproductiveSymbolsRemover::remove);
auto UnproductiveSymbolsRemoverLeftRG = UnproductiveSymbolsRemover::RegistratorWrapper<grammar::LeftRG, grammar::LeftRG>(UnproductiveSymbolsRemover::getInstance(), UnproductiveSymbolsRemover::remove);
auto UnproductiveSymbolsRemoverRightLG = UnproductiveSymbolsRemover::RegistratorWrapper<grammar::RightLG, grammar::RightLG>(UnproductiveSymbolsRemover::getInstance(), UnproductiveSymbolsRemover::remove);
auto UnproductiveSymbolsRemoverRightRG = UnproductiveSymbolsRemover::RegistratorWrapper<grammar::RightRG, grammar::RightRG>(UnproductiveSymbolsRemover::getInstance(), UnproductiveSymbolsRemover::remove);

grammar::Grammar UnproductiveSymbolsRemover::remove(const grammar::Grammar& grammar) {
	return getInstance().dispatch(grammar.getData());
}

} /* namespace simplify */

} /* namespace grammar */

