#ifndef GRAPH_DIJKSTRA_H_
#define GRAPH_DIJKSTRA_H_

#include <common/multipleDispatch.hpp>
#include <unordered_map>

#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph {

namespace shortestpath {

// Dijkstra only works on graphs without negative-weight edges (>= 0)

class Dijkstra : public std::SingleDispatchLastStaticParam<std::unordered_map<Node, int>, graph::GraphBase, const Node&> {
public:
	typedef std::unordered_map<Node, int> Result;

	static Result dijkstra(const Graph &graph, const Node &start);

	static Result dijkstra(const DirectedGraph &graph, const Node &start);
	static Result dijkstra(const UndirectedGraph &graph, const Node &start);

	static Dijkstra& getInstance() {
		static Dijkstra res;
		return res;
	}
};

} // namespace shortestpath

} // namespace graph

#endif // GRAPH_DIJKSTRA_H_
