#include "Bfs.h"

#include <queue>
#include <unordered_map>

namespace graph {

namespace traverse {

struct Data {
	Node start;
	Bfs::Function func;
	Bfs::FunctionExt func2;
};

template <typename T>
static void bfs_impl(const T &graph, const Node &start, Bfs::Function func) {
	std::unordered_map<Node, bool> visited;
	std::queue<Node> q;

	q.push(start);
	visited[start] = true;

	while (!q.empty()) {
		Node n = q.front(); q.pop();

		if (!func(n)) {
			return;
		}

		for (const Node &e : graph.neighbors(n)) {
			if (visited.find(e) == visited.end()) {
				visited[e] = true;
				q.push(e);
			}
		}
	}
}

template <typename T>
static void bfs2_impl(const T &graph, const Node &start, Bfs::FunctionExt func) {
	std::unordered_map<Node, int> distances;
	std::unordered_map<Node, Node> predecessors;
	std::queue<Node> q;

	q.push(start);
	distances[start] = 0;
	predecessors.insert(std::make_pair(start, start));

	while (!q.empty()) {
		Node n = q.front(); q.pop();

		if (!func(n, predecessors.find(n)->second, distances.find(n)->second)) {
			return;
		}

		for (const Node &e : graph.neighbors(n)) {
			if (distances.find(e) == distances.end()) {
				distances[e] = distances.at(n) + 1;
				predecessors.insert(std::make_pair(e, n));
				q.push(e);
			}
		}
	}
}

void Bfs::bfs(const Graph &graph, const Node &start, Bfs::Function func) {
	getInstance().dispatch(graph.getData(), start, func);
}

void Bfs::bfs(const Graph &graph, const Node &start, Bfs::FunctionExt func) {
	getInstance().dispatch(graph.getData(), start, func);
}

void Bfs::bfs(const DirectedGraph &graph, const Node &start, Bfs::Function func) {
	bfs_impl(graph, start, func);
}

auto BfsDirectedGraph1 = Bfs::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, Bfs::Function>::RegistratorWrapper<void, graph::DirectedGraph>(Bfs::getInstance(), Bfs::bfs);

void Bfs::bfs(const DirectedGraph &graph, const Node &start, Bfs::FunctionExt func) {
	bfs2_impl(graph, start, func);
}

auto BfsDirectedGraph2 = Bfs::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, Bfs::FunctionExt>::RegistratorWrapper<void, graph::DirectedGraph>(Bfs::getInstance(), Bfs::bfs);

void Bfs::bfs(const UndirectedGraph &graph, const Node &start, Bfs::Function func) {
	bfs_impl(graph, start, func);
}

auto BfsUndirectedGraph1 = Bfs::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, Bfs::Function>::RegistratorWrapper<void, graph::UndirectedGraph>(Bfs::getInstance(), Bfs::bfs);

void Bfs::bfs(const UndirectedGraph &graph, const Node &start, Bfs::FunctionExt func) {
	bfs2_impl(graph, start, func);
}

auto BfsUndirectedGraph2 = Bfs::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, Bfs::FunctionExt>::RegistratorWrapper<void, graph::UndirectedGraph>(Bfs::getInstance(), Bfs::bfs);

} // namespace traverse

} // namespace graph
