#ifndef GRAPH_BFS_H_
#define GRAPH_BFS_H_

#include <common/multipleDispatch.hpp>
#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph {

namespace traverse {

// func is called for each visited node, traversal stops when returning false
//
// bool Function(const Node &visitedNode);
// bool FunctionExt(const Node &visitedNode, const Node &predecessor, int distanceFromStart);

class Bfs : public std::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, std::function<bool(const Node&)>>, public std::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, std::function<bool(const Node&, const Node&, int)>> {
public:
	using std::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, std::function<bool(const Node&)>>::dispatch;
	using std::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, std::function<bool(const Node&, const Node&, int)>>::dispatch;

	typedef std::function<bool(const Node&)> Function;
	typedef std::function<bool(const Node&, const Node&, int)> FunctionExt;

	static void bfs(const Graph &graph, const Node &start, Function func);
	static void bfs(const Graph &graph, const Node &start, FunctionExt func);

	static void bfs(const DirectedGraph &graph, const Node &start, Function func);
	static void bfs(const DirectedGraph &graph, const Node &start, FunctionExt func);
	static void bfs(const UndirectedGraph &graph, const Node &start, Function func);
	static void bfs(const UndirectedGraph &graph, const Node &start, FunctionExt func);

	static Bfs& getInstance() {
		static Bfs res;
		return res;
	}
};

} // namespace traverse

} // namespace graph

#endif // GRAPH_BFS_H_
