/*
 * ToAutomatonGlushkov.cpp
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "ToAutomatonGlushkov.h"

#include "label/Label.h"
#include "label/LabelPairLabel.h"
#include "label/ObjectLabel.h"
#include <automaton/Automaton.h>

#include "object/Object.h"

#include "../GlushkovTraversal.h"
#include "../GlushkovPair.h"
#include "../GlushkovSymbol.h"

#include "../properties/RegExpEpsilon.h"

namespace regexp {

namespace convert {

automaton::NFA ToAutomatonGlushkov::convert(const regexp::RegExp& regexp) {
	return getInstance().dispatch(regexp.getData());
}

automaton::NFA ToAutomatonGlushkov::convert(const regexp::UnboundedRegExp& regexp) {
	automaton::State q0( label::Label( label::LabelPairLabel( std::make_pair( label::labelFrom( 'q' ), label::labelFrom( 0 ) ) ) ) );
	automaton::NFA automaton( q0 );

	// step 1
	automaton.setInputAlphabet(regexp.getAlphabet());

	// steps 2, 3, 4
	std::set<regexp::GlushkovPair> pairs;
	const std::set<regexp::GlushkovSymbol> first = regexp::GlushkovTraversal::first(regexp);
	const std::set<regexp::GlushkovSymbol> last = regexp::GlushkovTraversal::last(regexp);
	for( auto const& x : regexp::GlushkovTraversal::getSymbols( regexp ) )
		for( auto const& f : regexp::GlushkovTraversal::follow( regexp, x ) ) {
			pairs.insert( regexp::GlushkovPair( x, f ) );
		}
	// \e in q0 check is in step 7

	// step 5
	std::map<regexp::GlushkovSymbol, automaton::State> stateMap;

	for( auto const& symbol : regexp::GlushkovTraversal::getSymbols( regexp ) ) {
		automaton::State q( label::Label( label::LabelPairLabel( std::make_pair( label::Label( label::ObjectLabel( alib::Object( symbol.getInputSymbol( ).getData() ) ) ), label::labelFrom( symbol.getId( ) ) ) ) ) );

		stateMap.insert( std::make_pair( symbol, q ) );
		automaton.addState( q );
	}

	// step 6
	for( auto const& symbol : first ) {
		const automaton::State & q = stateMap.find( symbol )->second;

		automaton.addTransition( q0, symbol.getInputSymbol( ), q );
	}

	for( auto const& pair : pairs ) {
		const automaton::State & p = stateMap.find( pair.getFirst( ) )->second;
		const automaton::State & q = stateMap.find( pair.getSecond( ) )->second;

		automaton.addTransition( p, pair.getSecond( ).getInputSymbol( ), q );
	}

	// step 7
	for( auto const& symbol : last ) {
		const automaton::State & q = stateMap.find( symbol )->second;
		automaton.addFinalState( q );
	}

	if(regexp::properties::RegExpEpsilon::languageContainsEpsilon(regexp))
		automaton.addFinalState( q0 );

	return automaton;
}

auto ToAutomatonGlushkovUnboundedRegExp = ToAutomatonGlushkov::RegistratorWrapper<automaton::NFA, regexp::UnboundedRegExp>(ToAutomatonGlushkov::getInstance(), ToAutomatonGlushkov::convert);

automaton::NFA ToAutomatonGlushkov::convert(const regexp::FormalRegExp& /* regexp */)
{
	throw exception::AlibException("Glushkov: Converting FormalRegExp NYI"); // TODO
}

auto ToAutomatonGlushkovFormalRegExp = ToAutomatonGlushkov::RegistratorWrapper<automaton::NFA, regexp::FormalRegExp>(ToAutomatonGlushkov::getInstance(), ToAutomatonGlushkov::convert);

} /* namespace convert */

} /* namespace regexp */
