/*
 * RegExpIntegral.h
 *
 *  Created on: 24. 2. 2014
 *	  Author: Tomas Pecka
 */

#ifndef REGEXPINTEGRAL_H_
#define REGEXPINTEGRAL_H_

#include <common/multipleDispatch.hpp>

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

#include <string/LinearString.h>

namespace regexp
{

/**
 * Calculates integral of regular expression
 * Source: Melichar definition 2.93 in chapter 2.4.4
 */
class RegExpIntegral : public std::SingleDispatchLastStaticParam<regexp::RegExp, regexp::RegExpBase, const string::LinearString&>, regexp::FormalRegExpElement::const_visitor_type, regexp::UnboundedRegExpElement::const_visitor_type
{
public:
	RegExpIntegral() {}

	/**
	 * @param regexp RegExp to integrate
	 * @param string String to integrate given RegExp over
	 * @return integral of regexp
	 */
	static regexp::RegExp integral(const regexp::RegExp& regexp, const string::LinearString& string);

	static regexp::FormalRegExp integral(const regexp::FormalRegExp& regexp, const string::LinearString& string);
	static regexp::UnboundedRegExp integral(const regexp::UnboundedRegExp& regexp, const string::LinearString& string);

private:
	void Visit(void*, const regexp::FormalRegExpAlternation& alternation) const;
	void Visit(void*, const regexp::FormalRegExpConcatenation& concatenation) const;
	void Visit(void*, const regexp::FormalRegExpIteration& iteration) const;
	void Visit(void*, const regexp::FormalRegExpSymbol& symbol) const;
	void Visit(void*, const regexp::FormalRegExpEpsilon& epsilon) const;
	void Visit(void*, const regexp::FormalRegExpEmpty& empty) const;

	void Visit(void*, const regexp::UnboundedRegExpAlternation& alternation) const;
	void Visit(void*, const regexp::UnboundedRegExpConcatenation& concatenation) const;
	void Visit(void*, const regexp::UnboundedRegExpIteration& iteration) const;
	void Visit(void*, const regexp::UnboundedRegExpSymbol& symbol) const;
	void Visit(void*, const regexp::UnboundedRegExpEpsilon& epsilon) const;
	void Visit(void*, const regexp::UnboundedRegExpEmpty& empty) const;

	static const RegExpIntegral REGEXP_INTEGRAL;

public:
	static RegExpIntegral& getInstance() {
		static RegExpIntegral res;
		return res;
	}
};

} /* namespace regexp */

#endif /* REGEXPINTEGRAL_H_ */
