/*
 * RandomStringFactory.cpp
 *
 *  Created on: 27. 3. 2014
 *	  Author: Jan Travnicek
 */

#include "RandomStringFactory.h"

#include <algorithm>
#include <random>

namespace string {

namespace generate {

string::LinearString RandomStringFactory::generateLinearString( size_t size, size_t alphabetSize, bool randomizedAlphabet ) {
	srand( time( NULL ) );

	if(alphabetSize > 26)
		throw exception::AlibException("Too big alphabet.");

	if( alphabetSize <= 0 )
		throw exception::AlibException( "Alphabet size must be greater than 0." );

	std::vector<alphabet::Symbol> symbols;
	for(int i = 0; i < 26; i++) symbols.push_back(alphabet::symbolFrom(i + 'a'));
	if(randomizedAlphabet) shuffle(symbols.begin(), symbols.end(), std::random_devices::semirandom);
	std::set<alphabet::Symbol> alphabet(symbols.begin(), symbols.begin() + alphabetSize);

	return RandomStringFactory::generateLinearString( size, alphabet );
}

string::LinearString RandomStringFactory::generateLinearString( size_t size, std::set<alphabet::Symbol> alphabet) {

	if( alphabet.size() > 26)
		throw exception::AlibException("Too big alphabet.");

	if( alphabet.size() <= 0 )
		throw exception::AlibException( "Alphabet size must be greater than 0." );

	std::vector<alphabet::Symbol> alphabetList(alphabet.begin(), alphabet.end());
	std::vector<alphabet::Symbol> elems;

	for(size_t i = 0; i < size; i++) {
		elems.push_back(alphabetList[std::random_devices::semirandom() % alphabetList.size()]);
	}

	return string::LinearString(elems);
}

} /* namespace generate */

} /* namespace string */
