/*
 * BoyerMooreHorspool.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka
 */

#include "BoyerMooreHorspool.h"
#include "BadCharacterShiftTable.h"

#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <alphabet/Symbol.h>

#include <map>

namespace stringology {

namespace exact {

std::set<unsigned> BoyerMooreHorspool::match(const string::String& subject, const string::String& pattern) {
	return getInstance().dispatch(subject.getData(), pattern.getData());
}

std::set<unsigned> BoyerMooreHorspool::match(const string::LinearString& string, const string::LinearString& pattern)
{
	std::set<unsigned> occ;
	std::map<alphabet::Symbol, size_t> bcs = BadCharacterShiftTable::bcs(string.getAlphabet(), pattern);

	size_t haystack_offset = 0;
	while(haystack_offset + pattern.getContent().size() <= string.getContent().size()) {
		size_t i = pattern.getContent().size();
		while(i > 0 && string.getContent().at(haystack_offset + i - 1) == pattern.getContent().at(i - 1)) {
			i--;
		}

		// Yay, there is match!!!
		if(i == 0) occ.insert(haystack_offset);
		haystack_offset += bcs[string.getContent().at(haystack_offset + pattern.getContent().size() - 1)];
		//std::cout << haystack_offset << std::endl;
	}
	return occ;
}

auto BoyerMooreHorpoolLinearStringLinearString = BoyerMooreHorspool::RegistratorWrapper<std::set<unsigned>, string::LinearString, string::LinearString>(BoyerMooreHorspool::getInstance(), BoyerMooreHorspool::match);

} /* namespace exact */

} /* namespace stringology */
