/*
 * ExactNondeterministicSubsequenceAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactNondeterministicSubsequenceAutomaton.h"
#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <string/Epsilon.h>

#include <deque>

namespace stringology {

namespace exact {

automaton::Automaton ExactNondeterministicSubsequenceAutomaton::construct(const string::String& text) {
	return getInstance().dispatch(text.getData());
}

automaton::EpsilonNFA ExactNondeterministicSubsequenceAutomaton::construct(const string::LinearString& text) {
	automaton::EpsilonNFA res(automaton::State(0));
	res.addFinalState(automaton::State(0));
	res.setInputAlphabet(text.getAlphabet());

	int i = 1;
	for(const alphabet::Symbol& symbol : text.getContent()) {
		res.addState(automaton::State(i));
		res.addFinalState(automaton::State(i));

		res.addTransition(automaton::State(i-1), symbol, automaton::State(i));
		res.addTransition(automaton::State(i-1), automaton::State(i));
		i++;
	}
	return res;
}

auto ExactNondeterministicSubsequenceAutomatonLinearString = ExactNondeterministicSubsequenceAutomaton::RegistratorWrapper<automaton::EpsilonNFA, string::LinearString>(ExactNondeterministicSubsequenceAutomaton::getInstance(), ExactNondeterministicSubsequenceAutomaton::construct);

} /* namespace exact */

} /* namespace stringology */
