/*
 * SuffixTrie.h
 *
 *  Created on: 1. 11. 2014
 *      Author: Tomas Pecka
 */

#ifndef SUFFIX_TRIE_H_
#define SUFFIX_TRIE_H_

#include <indexes/suffixTrie/SuffixTrieFinalMark.h>
#include <indexes/suffixTrie/SuffixTrieTerminatingSymbol.h>
#include <string/String.h>
#include <string/LinearString.h>
#include <string/LinearStringTerminatingSymbol.h>
#include <common/multipleDispatch.hpp>

namespace stringology {

namespace indexing {

/**
 * Constructs suffix trie for given string.
 *
 * Source: Lectures MI-EVY (CTU in Prague), Year 2014, Lecture 3, slide 4
 */

class SuffixTrie : public std::SingleDispatch < indexes::SuffixTrieFinalMark, string::StringBase > {
public:
	/**
	 * Creates suffix trie
	 * @param string string to construct suffix trie for
	 * @return automaton
	 */
	static indexes::SuffixTrieFinalMark construct ( const string::String & string );

	static indexes::SuffixTrieFinalMark construct ( const string::LinearString & string );
	static indexes::SuffixTrieTerminatingSymbol construct ( const string::LinearStringTerminatingSymbol & string );

public:
	static SuffixTrie & getInstance ( ) {
		static SuffixTrie res;

		return res;
	}

};

} /* namespace indexing */

} /* namespace stringology */

#endif /* SUFFIX_TRIE_H_ */
