#include "DijkstraTest.h"

#include "graph/shortestpath/Dijkstra.h"

#include <exception/AlibException.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GraphDijkstraTest, "graph" );
CPPUNIT_TEST_SUITE_REGISTRATION( GraphDijkstraTest );

void GraphDijkstraTest::testSimple()
{
	// Common
	graph::shortestpath::Dijkstra::Result res;
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");
	graph::Node n6("n6");

	// Directed
	graph::DirectedGraph dg;
	dg.addEdge(graph::DirectedEdge(n1, n2), 2);
	dg.addEdge(graph::DirectedEdge(n1, n3), 2);
	dg.addEdge(graph::DirectedEdge(n1, n4), 1);
	dg.addEdge(graph::DirectedEdge(n1, n5), 6);
	dg.addEdge(graph::DirectedEdge(n2, n5), 3);
	dg.addEdge(graph::DirectedEdge(n5, n6), 2);

	res = graph::shortestpath::Dijkstra::dijkstra(dg, n1);

	CPPUNIT_ASSERT_EQUAL(0, res[n1]);
	CPPUNIT_ASSERT_EQUAL(2, res[n2]);
	CPPUNIT_ASSERT_EQUAL(2, res[n3]);
	CPPUNIT_ASSERT_EQUAL(1, res[n4]);
	CPPUNIT_ASSERT_EQUAL(5, res[n5]);
	CPPUNIT_ASSERT_EQUAL(7, res[n6]);

	// Undirected
	graph::UndirectedGraph ug;
	ug.addEdge(graph::UndirectedEdge(n1, n2), 2);
	ug.addEdge(graph::UndirectedEdge(n1, n3), 2);
	ug.addEdge(graph::UndirectedEdge(n1, n4), 1);
	ug.addEdge(graph::UndirectedEdge(n1, n5), 6);
	ug.addEdge(graph::UndirectedEdge(n2, n5), 3);
	ug.addEdge(graph::UndirectedEdge(n5, n6), 2);

	res = graph::shortestpath::Dijkstra::dijkstra(ug, n1);

	CPPUNIT_ASSERT_EQUAL(0, res[n1]);
	CPPUNIT_ASSERT_EQUAL(2, res[n2]);
	CPPUNIT_ASSERT_EQUAL(2, res[n3]);
	CPPUNIT_ASSERT_EQUAL(1, res[n4]);
	CPPUNIT_ASSERT_EQUAL(5, res[n5]);
	CPPUNIT_ASSERT_EQUAL(7, res[n6]);
}

void GraphDijkstraTest::testCycle()
{
	// Common
	graph::shortestpath::Dijkstra::Result res;
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");
	graph::Node n6("n6");

	// Directed
	graph::DirectedGraph dg;
	dg.addEdge(graph::DirectedEdge(n1, n2), 2);
	dg.addEdge(graph::DirectedEdge(n1, n3), 2);
	dg.addEdge(graph::DirectedEdge(n1, n4), 1);
	dg.addEdge(graph::DirectedEdge(n1, n5), 6);
	dg.addEdge(graph::DirectedEdge(n2, n5), 3);
	dg.addEdge(graph::DirectedEdge(n5, n6), 2);
	dg.addEdge(graph::DirectedEdge(n6, n1), 1);

	res = graph::shortestpath::Dijkstra::dijkstra(dg, n1);

	CPPUNIT_ASSERT_EQUAL(0, res[n1]);
	CPPUNIT_ASSERT_EQUAL(2, res[n2]);
	CPPUNIT_ASSERT_EQUAL(2, res[n3]);
	CPPUNIT_ASSERT_EQUAL(1, res[n4]);
	CPPUNIT_ASSERT_EQUAL(5, res[n5]);
	CPPUNIT_ASSERT_EQUAL(7, res[n6]);

	// Undirected
	graph::UndirectedGraph ug;
	ug.addEdge(graph::UndirectedEdge(n1, n2), 2);
	ug.addEdge(graph::UndirectedEdge(n1, n3), 2);
	ug.addEdge(graph::UndirectedEdge(n1, n4), 1);
	ug.addEdge(graph::UndirectedEdge(n1, n5), 6);
	ug.addEdge(graph::UndirectedEdge(n2, n5), 3);
	ug.addEdge(graph::UndirectedEdge(n5, n6), 2);
	ug.addEdge(graph::UndirectedEdge(n6, n1), 1);

	res = graph::shortestpath::Dijkstra::dijkstra(ug, n1);

	CPPUNIT_ASSERT_EQUAL(0, res[n1]);
	CPPUNIT_ASSERT_EQUAL(2, res[n2]);
	CPPUNIT_ASSERT_EQUAL(2, res[n3]);
	CPPUNIT_ASSERT_EQUAL(1, res[n4]);
	CPPUNIT_ASSERT_EQUAL(5, res[n5]);
	CPPUNIT_ASSERT_EQUAL(1, res[n6]);
}

void GraphDijkstraTest::testMultiEdge()
{
	// Common
	graph::shortestpath::Dijkstra::Result res;
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");
	graph::Node n6("n6");

	// Directed
	graph::DirectedGraph dg;
	dg.addEdge(graph::DirectedEdge(n1, n2), 2);
	dg.addEdge(graph::DirectedEdge(n1, n2, "multi"), 1);
	dg.addEdge(graph::DirectedEdge(n1, n3), 2);
	dg.addEdge(graph::DirectedEdge(n1, n4), 1);
	dg.addEdge(graph::DirectedEdge(n1, n5), 6);
	dg.addEdge(graph::DirectedEdge(n2, n5), 3);
	dg.addEdge(graph::DirectedEdge(n5, n6), 2);
	dg.addEdge(graph::DirectedEdge(n6, n1), 1);

	res = graph::shortestpath::Dijkstra::dijkstra(dg, n1);

	CPPUNIT_ASSERT_EQUAL(0, res[n1]);
	CPPUNIT_ASSERT_EQUAL(1, res[n2]);
	CPPUNIT_ASSERT_EQUAL(2, res[n3]);
	CPPUNIT_ASSERT_EQUAL(1, res[n4]);
	CPPUNIT_ASSERT_EQUAL(4, res[n5]);
	CPPUNIT_ASSERT_EQUAL(6, res[n6]);

	// Undirected
	graph::UndirectedGraph ug;
	ug.addEdge(graph::UndirectedEdge(n1, n2), 2);
	ug.addEdge(graph::UndirectedEdge(n1, n2, "multi"), 1);
	ug.addEdge(graph::UndirectedEdge(n1, n3), 2);
	ug.addEdge(graph::UndirectedEdge(n1, n4), 1);
	ug.addEdge(graph::UndirectedEdge(n1, n5), 6);
	ug.addEdge(graph::UndirectedEdge(n2, n5), 3);
	ug.addEdge(graph::UndirectedEdge(n5, n6), 2);

	res = graph::shortestpath::Dijkstra::dijkstra(ug, n1);

	CPPUNIT_ASSERT_EQUAL(0, res[n1]);
	CPPUNIT_ASSERT_EQUAL(1, res[n2]);
	CPPUNIT_ASSERT_EQUAL(2, res[n3]);
	CPPUNIT_ASSERT_EQUAL(1, res[n4]);
	CPPUNIT_ASSERT_EQUAL(4, res[n5]);
	CPPUNIT_ASSERT_EQUAL(6, res[n6]);
}

void GraphDijkstraTest::testNegativeValue()
{
	// Common
	bool exception;
	graph::shortestpath::Dijkstra::Result res;
	graph::Node n1("n1");
	graph::Node n2("n2");

	// Directed
	graph::DirectedGraph dg;
	dg.addEdge(graph::DirectedEdge(n1, n2), -2);

	exception = false;
	try {
		res = graph::shortestpath::Dijkstra::dijkstra(dg, n1);
	} catch (exception::AlibException) {
		exception = true;
	}

	CPPUNIT_ASSERT_EQUAL(true, exception);

	// Undirected
	graph::UndirectedGraph ug;
	ug.addEdge(graph::UndirectedEdge(n1, n2), -2);

	exception = false;
	try {
		res = graph::shortestpath::Dijkstra::dijkstra(ug, n1);
	} catch (exception::AlibException) {
		exception = true;
	}

	CPPUNIT_ASSERT_EQUAL(true, exception);
}
