/*
 * XmlApi.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef XML_API_HPP_
#define XML_API_HPP_

#include <functional>
#include <deque>
#include <map>
#include <string>
#include "sax/Token.h"
#include "sax/FromXMLParserHelper.h"

#include "object/ObjectBase.h"
#include "object/WrapperBase.h"

namespace alib {

template < typename T, typename Enable = void >
struct xmlApi { };

template < typename Group >
struct xmlApi < Group, typename std::enable_if < std::is_base_of < WrapperBase, Group >::value >::type > {
	class ParserRegisterBase {
	public:
		virtual Group parse ( std::deque < sax::Token >::iterator & input ) = 0;
	};

private:
	 // INFO: Function exist to handle static order of initialisation
	static std::map < std::string, typename xmlApi < Group >::ParserRegisterBase * > & parseFunctions ( ) {
		static std::map < std::string, typename xmlApi < Group >::ParserRegisterBase * > res;

		return res;
	}

public:
	template < class Type >
	class ParserRegister : public ParserRegisterBase {
		std::function < Type ( std::deque < sax::Token >::iterator & ) > parseFunction;

	public:
		ParserRegister ( std::string tagName, std::function < Type ( std::deque < sax::Token >::iterator & ) > parseFunction ) : parseFunction ( parseFunction ) {
			parseFunctions ( ).insert ( std::make_pair ( std::move ( tagName ), this ) );
		}

		virtual Group parse ( std::deque < sax::Token >::iterator & input ) {
			return Group ( parseFunction ( input ) );
		}

	};

	static Group parse ( std::deque < sax::Token >::iterator & input ) {
		const std::string & tagName = sax::FromXMLParserHelper::getTokenData ( input, sax::Token::TokenType::START_ELEMENT );

		typename std::map < std::string, ParserRegisterBase * >::iterator callback = parseFunctions ( ).find ( tagName );

		if ( callback == parseFunctions ( ).end ( ) ) throw std::bad_function_call ( );

		return callback->second->parse ( input );
	}

	static bool first ( const std::deque < sax::Token >::const_iterator & input ) {
		const std::string & tagName = sax::FromXMLParserHelper::getTokenData ( input, sax::Token::TokenType::START_ELEMENT );

		typename std::map < std::string, ParserRegisterBase * >::iterator callback = parseFunctions ( ).find ( tagName );

		if ( callback == parseFunctions ( ).end ( ) ) return false;

		return true;
	}

	static void compose ( std::deque < sax::Token > & output, const Group & data ) {
		data.getData ( ).compose ( output );
	}

};

template < typename Type >
struct xmlApi < Type, typename std::enable_if < std::is_base_of < ObjectBase, Type >::value >::type > {
	static Type parse ( std::deque < sax::Token >::iterator & input ) {
		return Type::parse ( input );
	}

	static bool first ( const std::deque < sax::Token >::const_iterator & input ) {
		return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, Type::XML_TAG_NAME );
	}

	static void compose ( std::deque < sax::Token > & output, const Type & data ) {
		data.compose ( output );
	}

};

} /* namespace alib */

#endif /* XML_API_HPP_ */
