/*
 * BarSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef BAR_SYMBOL_H_
#define BAR_SYMBOL_H_

#include "SymbolBase.h"

namespace alphabet {

/**
 * Represents bar symbol for tree linearization.
 */
class BarSymbol : public SymbolBase {
public:
	/**
	 * Creates a bar symbol.
	 * @param symbol name of the symbol
	 */
	explicit BarSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const BarSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static BarSymbol BAR;

	const static std::string XML_TAG_NAME;

	static BarSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

namespace std {

template < >
struct compare < alphabet::BarSymbol > {
	int operator ()( const alphabet::BarSymbol & first, const alphabet::BarSymbol & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* BAR_SYMBOL_H_ */
