/*
 * BottomOfTheStackSymbol.h
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#ifndef BOTTOM_OF_THE_STACK_SYMBOL_H_
#define BOTTOM_OF_THE_STACK_SYMBOL_H_

#include "SymbolBase.h"

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class BottomOfTheStackSymbol : public SymbolBase {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit BottomOfTheStackSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const BottomOfTheStackSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static BottomOfTheStackSymbol BOTTOM_OF_THE_STACK;

	const static std::string XML_TAG_NAME;

	static BottomOfTheStackSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

namespace std {

template < >
struct compare < alphabet::BottomOfTheStackSymbol > {
	int operator ()( const alphabet::BottomOfTheStackSymbol & first, const alphabet::BottomOfTheStackSymbol & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* BOTTOM_OF_THE_STACK_SYMBOL_H_ */
