/*
 * StartSymbol.cpp
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#include "StartSymbol.h"
#include "../sax/FromXMLParserHelper.h"
#include "Symbol.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"
#include "UniqueSymbol.h"

namespace alphabet {

StartSymbol::StartSymbol() {

}

SymbolBase* StartSymbol::clone() const {
	return new StartSymbol(*this);
}

SymbolBase* StartSymbol::plunder() && {
	return new StartSymbol(std::move(*this));
}

int StartSymbol::compare(const StartSymbol&) const {
	return 0;
}

void StartSymbol::operator>>(std::ostream& out) const {
	out << "(StartSymbol)";
}

StartSymbol::operator std::string () const {
	return "$";
}

StartSymbol StartSymbol::START = StartSymbol();

const std::string StartSymbol::XML_TAG_NAME = "StartSymbol";

StartSymbol StartSymbol::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, StartSymbol::XML_TAG_NAME);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, StartSymbol::XML_TAG_NAME);
	return StartSymbol();
}

void StartSymbol::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(StartSymbol::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(StartSymbol::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* StartSymbol::next() const {
	return new UniqueSymbol(Symbol(*this), primitive::Integer(0));
}

SymbolBase* StartSymbol::inc() && {
	return new UniqueSymbol(Symbol(std::move(*this)), primitive::Integer(0));
}

} /* namespace alphabet */

namespace alib {

xmlApi<alphabet::Symbol>::ParserRegister<alphabet::StartSymbol> startSymbolParserRegister = xmlApi<alphabet::Symbol>::ParserRegister<alphabet::StartSymbol>(alphabet::StartSymbol::XML_TAG_NAME, alphabet::StartSymbol::parse);
xmlApi<alib::Object>::ParserRegister<alphabet::StartSymbol> startSymbolParserRegister2 = xmlApi<alib::Object>::ParserRegister<alphabet::StartSymbol>(alphabet::StartSymbol::XML_TAG_NAME, alphabet::StartSymbol::parse);

} /* namespace alib */
