/*
 * SubtreeWildcardSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef SUBTREE_WILDCARD_SYMBOL_H_
#define SUBTREE_WILDCARD_SYMBOL_H_

#include "SymbolBase.h"

namespace alphabet {

/**
 * Represents subtreeWildcard symbol for tree linearization.
 */
class SubtreeWildcardSymbol : public SymbolBase {
public:
	/**
	 * Creates a subtreeWildcard symbol.
	 * @param symbol name of the symbol
	 */
	explicit SubtreeWildcardSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const SubtreeWildcardSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static SubtreeWildcardSymbol SUBTREE_WILDCARD;

	const static std::string XML_TAG_NAME;

	static SubtreeWildcardSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

namespace std {

template < >
struct compare < alphabet::SubtreeWildcardSymbol > {
	int operator ()( const alphabet::SubtreeWildcardSymbol & first, const alphabet::SubtreeWildcardSymbol & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* SUBTREE_WILDCARD_SYMBOL_H_ */
