/*
 * SymbolPairSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "SymbolPairSymbol.h"
#include <sstream>
#include "../sax/FromXMLParserHelper.h"
#include "Symbol.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"
#include "UniqueSymbol.h"

namespace alphabet {

SymbolPairSymbol::SymbolPairSymbol(std::pair<Symbol, Symbol> symbol) : symbol(std::move(symbol)) {

}

SymbolBase* SymbolPairSymbol::clone() const {
	return new SymbolPairSymbol(*this);
}

SymbolBase* SymbolPairSymbol::plunder() && {
	return new SymbolPairSymbol(std::move(*this));
}

const std::pair<Symbol, Symbol>& SymbolPairSymbol::getData() const {
	return symbol;
}

int SymbolPairSymbol::compare(const SymbolPairSymbol& other) const {
	int res = symbol.first.getData().compare(other.symbol.first.getData());
	if(res == 0) res = symbol.second.getData().compare(other.symbol.second.getData());
	return res;
}

void SymbolPairSymbol::operator>>(std::ostream& out) const {
	out << "(SymbolPairSymbol " << symbol << ")";
}

SymbolPairSymbol::operator std::string() const {
	std::stringstream ss;
	ss << "<";
	ss << (std::string) symbol.first;
	ss << ",";
	ss << (std::string) symbol.second;
	ss << "}";
	return std::move(ss).str();
}

const std::string SymbolPairSymbol::XML_TAG_NAME = "SymbolPairSymbol";

SymbolPairSymbol SymbolPairSymbol::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, SymbolPairSymbol::XML_TAG_NAME);

	Symbol firstSymbol = alib::xmlApi<Symbol>::parse(input);
	Symbol secondSymbol = alib::xmlApi<Symbol>::parse(input);

	SymbolPairSymbol data(std::make_pair(std::move(firstSymbol), std::move(secondSymbol)));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, SymbolPairSymbol::XML_TAG_NAME);
	return data;
}

void SymbolPairSymbol::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(SymbolPairSymbol::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<Symbol>::compose(out, symbol.first);
	alib::xmlApi<Symbol>::compose(out, symbol.second);
	out.emplace_back(SymbolPairSymbol::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* SymbolPairSymbol::next() const {
	return new UniqueSymbol(Symbol(*this), primitive::Integer(0));
}

SymbolBase* SymbolPairSymbol::inc() && {
	return new UniqueSymbol(Symbol(std::move(*this)), primitive::Integer(0));
}

} /* namespace alphabet */

namespace alib {

xmlApi<alphabet::Symbol>::ParserRegister<alphabet::SymbolPairSymbol> symbolPairSymbolParserRegister = xmlApi<alphabet::Symbol>::ParserRegister<alphabet::SymbolPairSymbol>(alphabet::SymbolPairSymbol::XML_TAG_NAME, alphabet::SymbolPairSymbol::parse);
xmlApi<alib::Object>::ParserRegister<alphabet::SymbolPairSymbol> symbolPairSymbolParserRegister2 = xmlApi<alib::Object>::ParserRegister<alphabet::SymbolPairSymbol>(alphabet::SymbolPairSymbol::XML_TAG_NAME, alphabet::SymbolPairSymbol::parse);

} /* namespace alib */
