/*
 * VariablesBarSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef VARIABLES_BAR_SYMBOL_H_
#define VARIABLES_BAR_SYMBOL_H_

#include "SymbolBase.h"

namespace alphabet {

/**
 * Represents bar symbol for tree linearization.
 */
class VariablesBarSymbol : public SymbolBase {
public:
	/**
	 * Creates a bar symbol.
	 * @param symbol name of the symbol
	 */
	explicit VariablesBarSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const VariablesBarSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static VariablesBarSymbol BAR;

	const static std::string XML_TAG_NAME;

	static VariablesBarSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * next ( ) const;
	virtual SymbolBase * inc ( ) &&;
};

} /* namespace alphabet */

namespace std {

template < >
struct compare < alphabet::VariablesBarSymbol > {
	int operator ()( const alphabet::VariablesBarSymbol & first, const alphabet::VariablesBarSymbol & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* VARIABLES_BAR_SYMBOL_H_ */
