/*
 * AutomatonFromXMLParser.h
 *
 *  Created on: Oct 12, 2013
 *      Author: Martin Zak
 */

#ifndef AUTOMATON_FROM_XML_PARSER_H_
#define AUTOMATON_FROM_XML_PARSER_H_

#include "../../sax/FromXMLParserHelper.h"
#include <set>
#include <vector>
#include <variant>
#include <map>
#include "State.h"
#include "../../regexp/RegExpFeatures.h"
#include "../../alphabet/SymbolFeatures.h"
#include "../../string/StringFeatures.h"
#include "Shift.h"

namespace automaton {

/**
 * Parser used to get general FSM or EpsilonNFA, NFA, DFA from XML parsed into deque of Tokens.
 */
class AutomatonFromXMLParser {
public:
	static std::set<State> parseStates(std::deque<sax::Token>::iterator& input);
	static State parseState(std::deque<sax::Token>::iterator& input);
	static std::set<alphabet::Symbol> parseInputAlphabet(std::deque<sax::Token>::iterator& input);
	static std::set<alphabet::Symbol> parseCallInputAlphabet(std::deque<sax::Token>::iterator& input);
	static std::set<alphabet::Symbol> parseReturnInputAlphabet(std::deque<sax::Token>::iterator& input);
	static std::set<alphabet::Symbol> parseLocalInputAlphabet(std::deque<sax::Token>::iterator& input);
	static std::set<alphabet::RankedSymbol> parseRankedInputAlphabet(std::deque<sax::Token>::iterator& input);
	static std::set<alphabet::Symbol> parseStackAlphabet(std::deque<sax::Token>::iterator& input);
	static std::set<alphabet::Symbol> parseInitialStackSymbols(std::deque<sax::Token>::iterator& input);
	static alphabet::Symbol parseInitialStackSymbol(std::deque<sax::Token>::iterator& input);
	static std::set<alphabet::Symbol> parseTapeAlphabet(std::deque<sax::Token>::iterator& input);
	static State parseInitialState(std::deque<sax::Token>::iterator& input);
	static std::set<State> parseInitialStates(std::deque<sax::Token>::iterator& input);
	static std::set<State> parseFinalStates(std::deque<sax::Token>::iterator& input);
	static alphabet::Symbol parseBlankSymbol(std::deque<sax::Token>::iterator& input);
	static alphabet::Symbol parseBottomOfTheStackSymbol(std::deque<sax::Token>::iterator& input);
	static std::map<alphabet::Symbol, std::pair<std::vector<alphabet::Symbol>, std::vector<alphabet::Symbol> > > parseInputToPushdownStoreOperation(std::deque<sax::Token>::iterator& input);

	static State parseTransitionFrom(std::deque<sax::Token>::iterator& input);
	static std::vector<State> parseTransitionFromMultiple(std::deque<sax::Token>::iterator& input);
	static State parseTransitionTo(std::deque<sax::Token>::iterator& input);
	static Shift parseTransitionShift(std::deque<sax::Token>::iterator& input);
	static std::vector<alphabet::Symbol> parseTransitionPop(std::deque<sax::Token>::iterator& input);
	static alphabet::Symbol parseTransitionSinglePop(std::deque<sax::Token>::iterator& input);
	static std::vector<alphabet::Symbol> parseTransitionPush(std::deque<sax::Token>::iterator& input);
	static alphabet::Symbol parseTransitionSinglePush(std::deque<sax::Token>::iterator& input);

	template<class T>
	static void parseTransitions(std::deque<sax::Token>::iterator& input, T& automaton);

	static alphabet::Symbol parseTransitionInputSymbol(std::deque<sax::Token>::iterator& input);
	static alphabet::RankedSymbol parseTransitionInputRankedSymbol(std::deque<sax::Token>::iterator& input);
	static std::variant<string::Epsilon, alphabet::Symbol> parseTransitionInputEpsilonSymbol(std::deque<sax::Token>::iterator& input);
	static std::variant<string::Epsilon, alphabet::Symbol> parseTransitionInputBlankEpsilonSymbol(std::deque<sax::Token>::iterator& input);
	static string::LinearString parseTransitionInputString(std::deque<sax::Token>::iterator& input);
	static regexp::RegExp parseTransitionInputRegexp(std::deque<sax::Token>::iterator& input);

	static alphabet::Symbol parseTransitionOutputSymbol(std::deque<sax::Token>::iterator& input);
	static std::variant<string::Epsilon, alphabet::Symbol> parseTransitionOutputEpsilonSymbol(std::deque<sax::Token>::iterator& input);
	static std::variant<string::Epsilon, alphabet::Symbol> parseTransitionOutputBlankEpsilonSymbol(std::deque<sax::Token>::iterator& input);

	static alphabet::Symbol parseSymbolIO(std::deque<sax::Token>::iterator& input);
	static std::variant<string::Epsilon, alphabet::Symbol> parseEpsilonSymbolIO(std::deque<sax::Token>::iterator& input);

};

template<class T>
void AutomatonFromXMLParser::parseTransitions(std::deque<sax::Token>::iterator& input, T& automaton) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "transitions");
	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		T::parseTransition(input, automaton);
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "transitions");
}

} /* namespace automaton */

#endif /* AUTOMATON_FROM_XML_PARSER_H_ */

