/*
 * AutomatonToXMLComposer.h
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_TO_XML_COMPOSER_H_
#define AUTOMATON_TO_XML_COMPOSER_H_

#include <string>
#include <deque>
#include <vector>
#include <set>
#include <variant>
#include <map>
#include "../AutomatonFeatures.h"
#include "../../regexp/RegExpFeatures.h"
#include "../../alphabet/SymbolFeatures.h"
#include "../../string/StringFeatures.h"
#include "../../sax/Token.h"

namespace automaton {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class AutomatonToXMLComposer {
public:
	static void composeStates(std::deque<sax::Token>&, const std::set<State>& states);
	static void composeInputAlphabet(std::deque<sax::Token>&, const std::set<alphabet::Symbol>& symbols);
	static void composeCallInputAlphabet(std::deque<sax::Token>&, const std::set<alphabet::Symbol>& symbols);
	static void composeReturnInputAlphabet(std::deque<sax::Token>&, const std::set<alphabet::Symbol>& symbols);
	static void composeLocalInputAlphabet(std::deque<sax::Token>&, const std::set<alphabet::Symbol>& symbols);
	static void composeRankedInputAlphabet(std::deque<sax::Token>&, const std::set<alphabet::RankedSymbol>& symbols);
	static void composeInitialStates(std::deque<sax::Token>&, const std::set<State>& states);
	static void composeInitialState(std::deque<sax::Token>&, const State& state);
	static void composeFinalStates(std::deque<sax::Token>&, const std::set<State>& states);
	static void composeStackAlphabet(std::deque<sax::Token>&, const std::set<alphabet::Symbol>& symbols);
	static void composeInitialStackSymbols(std::deque<sax::Token>&, const std::set<alphabet::Symbol>& symbols);
	static void composeInitialStackSymbol(std::deque<sax::Token>&, const alphabet::Symbol& symbols);
	static void composeTapeAlphabet(std::deque<sax::Token>&, const std::set<alphabet::Symbol>& symbols);
	static void composeBlankSymbol(std::deque<sax::Token>&, const alphabet::Symbol& symbol);
	static void composeBottomOfTheStackSymbol(std::deque<sax::Token>&, const alphabet::Symbol& symbol);
	static void composeInputToPushdownStoreOperation(std::deque<sax::Token>&, const std::map<alphabet::Symbol, std::pair<std::vector<alphabet::Symbol>, std::vector<alphabet::Symbol> > >& operations);

	static void composeTransitionTo(std::deque<sax::Token>&, const State& state);
	static void composeTransitionFrom(std::deque<sax::Token>&, const State& state);
	static void composeTransitionFrom(std::deque<sax::Token>&, const std::vector<State> & states);
	static void composeTransitionShift(std::deque<sax::Token>&, Shift shift);
	static void composeTransitionPop(std::deque<sax::Token>&, const std::vector<alphabet::Symbol>& symbols);
	static void composeTransitionSinglePop(std::deque<sax::Token>&, const alphabet::Symbol& symbol);
	static void composeTransitionPush(std::deque<sax::Token>&, const std::vector<alphabet::Symbol>& symbols);
	static void composeTransitionSinglePush(std::deque<sax::Token>&, const alphabet::Symbol& symbol);
	static void composeTransitionInputSymbol(std::deque<sax::Token>&, const alphabet::Symbol& symbol);
	static void composeTransitionInputSymbol(std::deque<sax::Token>&, const alphabet::RankedSymbol& symbol);
	static void composeTransitionInputEpsilonSymbol(std::deque<sax::Token>&, const std::variant<string::Epsilon, alphabet::Symbol>&);
	static void composeTransitionOutputSymbol(std::deque<sax::Token>&, const alphabet::Symbol& symbol);
	static void composeTransitionOutputEpsilonSymbol(std::deque<sax::Token>&, const std::variant<string::Epsilon, alphabet::Symbol>&);
	static void composeTransitionInputString(std::deque<sax::Token>&, const string::LinearString& string);
	static void composeTransitionInputRegexp(std::deque<sax::Token>&, const regexp::RegExp& regexp);
};

} /* namespace automaton */

#endif /* AUTOMATON_TO_XML_COMPOSER_H_ */

