/*
 * CallReturnLocalInputAlphabet.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef CALL_RETURN_LOCAL_INPUT_ALPHABET_H_
#define CALL_RETURN_LOCAL_INPUT_ALPHABET_H_

#include <set>
#include "../../alphabet/Symbol.h"


namespace automaton {

/**
 * Abstract base class for all automatons. Contains common elements of automatons.
 */
class CallReturnLocalInputAlphabet {
protected:
	std::set<alphabet::Symbol> callInputAlphabet;
	std::set<alphabet::Symbol> returnInputAlphabet;
	std::set<alphabet::Symbol> localInputAlphabet;
public:
	/**
	 * Adds input symbol to input alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol already exists
	 */
	bool addCallInputSymbol(alphabet::Symbol symbol);

	/**
	 * Sets input symbols of the automaton.
	 * @param symbols Symbols to set
	 * @throws AutomatonException when symbol already exists
	 */
	void setCallInputAlphabet(std::set<alphabet::Symbol> symbols);

	/**
	 * Adds input symbol to input alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol already exists
	 */
	bool addReturnInputSymbol(alphabet::Symbol symbol);

	/**
	 * Sets input symbols of the automaton.
	 * @param symbols Symbols to set
	 * @throws AutomatonException when symbol already exists
	 */
	void setReturnInputAlphabet(std::set<alphabet::Symbol> symbols);

	/**
	 * Adds input symbol to input alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol already exists
	 */
	bool addLocalInputSymbol(alphabet::Symbol symbol);

	/**
	 * Sets input symbols of the automaton.
	 * @param symbols Symbols to set
	 * @throws AutomatonException when symbol already exists
	 */
	void setLocalInputAlphabet(std::set<alphabet::Symbol> symbols);

	/**
	 * Removes input symbol from the input alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is not present in input alphabet
	 * or when symbol is part of the transition
	 */
	virtual bool removeInputSymbol(const alphabet::Symbol& symbol) = 0;

	/**
	 * @return the input alphabet
	 */
	const std::set<alphabet::Symbol>& getCallInputAlphabet() const;

	/**
	 * @return the input alphabet
	 */
	const std::set<alphabet::Symbol>& getReturnInputAlphabet() const;

	/**
	 * @return the input alphabet
	 */
	const std::set<alphabet::Symbol>& getLocalInputAlphabet() const;

};

} /* namespace automaton */

#endif /* CALL_RETURN_LOCAL_INPUT_ALPHABET_H_ */

