/*
 * MultiInitialSymbols.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#include "MultiInitialSymbolsPushdownStoreAlphabet.h"
#include "../AutomatonException.h"
#include <algorithm>

namespace automaton {

bool MultiInitialSymbolsPushdownStoreAlphabet::addInitialSymbol(alphabet::Symbol start) {
	if (!stackAlphabet.count(start)) {
		throw AutomatonException("Initial symbol \"" + (std::string) start + "\" not in a pushdown store alphabet.");
	}

	return this->initialSymbols.insert(std::move(start)).second;
}

bool MultiInitialSymbolsPushdownStoreAlphabet::removeInitialSymbol(const alphabet::Symbol& start) {
	return this->initialSymbols.erase(start);
}

void MultiInitialSymbolsPushdownStoreAlphabet::setInitialSymbols(std::set<alphabet::Symbol> symbols) {
	if(!std::includes(stackAlphabet.begin(), stackAlphabet.end(), symbols.begin(), symbols.end())) {
		throw AutomatonException("Initial symbol ? not in a pushdown store alphabet.");
	}

	this->initialSymbols = std::move(symbols);
}

const std::set<alphabet::Symbol>& MultiInitialSymbolsPushdownStoreAlphabet::getInitialSymbols() const {
	return initialSymbols;
}

} /* namespace automaton */
