/*
 * Base.h
 *
 *  Created on: Apr 05, 2014
 *      Author: Jan Travnicek
 */

#ifndef BASE_H_
#define BASE_H_

#include <typeinfo>
#include <typeindex>
#include <ostream>
#include <stdexcept>

namespace alib {

template < typename T >
class base {
public:
	virtual ~base ( ) noexcept { }

	virtual T * clone ( ) const = 0;

	virtual T * plunder ( ) && = 0;

	bool operator !=( const T & other ) const {
		return this->compare ( other ) != 0;
	}

	bool operator ==( const T & other ) const {
		return this->compare ( other ) == 0;
	}

	bool operator <( const T & other ) const {
		return this->compare ( other ) < 0;
	}

	bool operator <=( const T & other ) const {
		return this->compare ( other ) <= 0;
	}

	bool operator >( const T & other ) const {
		return this->compare ( other ) > 0;
	}

	bool operator >=( const T & other ) const {
		return this->compare ( other ) >= 0;
	}

	virtual int compare ( const T & other ) const = 0;

	friend std::ostream & operator <<( std::ostream & os, const T & instance ) {
		instance >> os;
		return os;
	}

	virtual void operator >>( std::ostream & ) const = 0;

	virtual explicit operator std::string ( ) const = 0;
};

} /* namespace alib */

#endif /* BASE_H_ */
