/*
 * GrammarFromXMLParser.h
 *
 *  Created on: Oct 12, 2013
 *      Author: Martin Zak
 */

#ifndef GRAMMAR_FROM_XML_PARSER_H_
#define GRAMMAR_FROM_XML_PARSER_H_

#include <deque>
#include <set>
#include <variant>
#include <vector>
#include "../../alphabet/SymbolFeatures.h"
#include "../../sax/Token.h"
#include "../../sax/FromXMLParserHelper.h"

namespace grammar {

/**
 * Parser used to get general FSM or EpsilonNFA, NFA, DFA from XML parsed into list of Tokens.
 */
class GrammarFromXMLParser {
public:
	static std::set<alphabet::Symbol> parseNonterminalAlphabet(std::deque<sax::Token>::iterator& input);
	static std::set<alphabet::Symbol> parseTerminalAlphabet(std::deque<sax::Token>::iterator& input);
	static alphabet::Symbol parseInitialSymbol(std::deque<sax::Token>::iterator& input);
	static bool parseGeneratesEpsilon(std::deque<sax::Token>::iterator& input);

	static std::vector<alphabet::Symbol> parseRuleLContext(std::deque<sax::Token>::iterator& input);
	static std::vector<alphabet::Symbol> parseRuleLHS(std::deque<sax::Token>::iterator& input);
	static std::vector<alphabet::Symbol> parseRuleRContext(std::deque<sax::Token>::iterator& input);
	static alphabet::Symbol parseRuleSingleSymbolLHS(std::deque<sax::Token>::iterator& input);
	static std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>> parseRuleGNFRHS(std::deque<sax::Token>::iterator& input);
	static std::vector<alphabet::Symbol> parseRuleRHS(std::deque<sax::Token>::iterator& input);
	static std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>> parseRuleOneOrTwoSymbolsRHS(std::deque<sax::Token>::iterator& input);

	template<class T>
	static void parseRules(std::deque<sax::Token>::iterator& input, T& grammar);
};

template<class T>
void GrammarFromXMLParser::parseRules(std::deque<sax::Token>::iterator& input, T& grammar) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "rules");
	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "rule");
		T::parseRule(input, grammar);
		sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "rule");
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "rules");
}

} /* namespace grammar */

#endif /* GRAMMAR_FROM_XML_PARSER_H_ */

