/*
 * TerminalNonterminalAlphabetAlphabetInitialSymbol.cpp
 *
 *  Created on: Nov 2, 2013
 *      Author: Martin Zak
 */

#include "TerminalNonterminalAlphabetInitialSymbol.h"
#include "../GrammarException.h"
#include "../../alphabet/LabeledSymbol.h"
#include "../../label/Label.h"
#include "../../alphabet/LabeledSymbol.h"
#include "../../alphabet/Symbol.h"
#include <climits>
#include <algorithm>
#include <iostream>

#include "../../XmlApi.hpp"

namespace grammar {

TerminalNonterminalAlphabetInitialSymbol::TerminalNonterminalAlphabetInitialSymbol(alphabet::Symbol initialSymbol) : initialSymbol(initialSymbol) {
	addNonterminalSymbol(std::move(initialSymbol));
}

bool TerminalNonterminalAlphabetInitialSymbol::addTerminalSymbol(alphabet::Symbol symbol) {
	if(nonterminalAlphabet.find(symbol) != nonterminalAlphabet.end()){
		throw GrammarException("Symbol \"" + (std::string) symbol + "\" is nonterminal symbol.");
	}

	return terminalAlphabet.insert(std::move(symbol)).second;
}

const std::set<alphabet::Symbol>& TerminalNonterminalAlphabetInitialSymbol::getTerminalAlphabet() const {
	return terminalAlphabet;
}

void TerminalNonterminalAlphabetInitialSymbol::setTerminalAlphabet(std::set<alphabet::Symbol> alphabet) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(terminalAlphabet.begin(), terminalAlphabet.end(), alphabet.begin(), alphabet.end(), std::inserter(removed, removed.end()));

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeTerminalSymbol(removedSymbol);
	}

	if(!std::excludes(terminalAlphabet.begin(), terminalAlphabet.end(), nonterminalAlphabet.begin(), nonterminalAlphabet.end()))
		throw GrammarException("Terminal alphabet and nonterminal alphabet intersect");

	terminalAlphabet = std::move(alphabet);
}

bool TerminalNonterminalAlphabetInitialSymbol::addNonterminalSymbol(alphabet::Symbol symbol) {
	if(terminalAlphabet.find(symbol) != terminalAlphabet.end()){
		throw GrammarException("Symbol \"" + (std::string) symbol + "\" is terminal symbol.");
	}

	return nonterminalAlphabet.insert(std::move(symbol)).second;
}

const std::set<alphabet::Symbol>& TerminalNonterminalAlphabetInitialSymbol::getNonterminalAlphabet() const {
	return nonterminalAlphabet;
}

void TerminalNonterminalAlphabetInitialSymbol::setNonterminalAlphabet(std::set<alphabet::Symbol> alphabet) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(nonterminalAlphabet.begin(), nonterminalAlphabet.end(), alphabet.begin(), alphabet.end(), std::inserter(removed, removed.end()));

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeNonterminalSymbol(removedSymbol);
	}

	if(!std::excludes(terminalAlphabet.begin(), terminalAlphabet.end(), nonterminalAlphabet.begin(), nonterminalAlphabet.end()))
		throw GrammarException("Terminal alphabet and nonterminal alphabet intersect");

	nonterminalAlphabet = std::move(alphabet);
}

const alphabet::Symbol& TerminalNonterminalAlphabetInitialSymbol::getInitialSymbol() const{
	return initialSymbol;
}

void TerminalNonterminalAlphabetInitialSymbol::setInitialSymbol(alphabet::Symbol symbol) {
	if(nonterminalAlphabet.find(symbol) == nonterminalAlphabet.end()) {
		throw GrammarException("Symbol \"" + (std::string) symbol + "\" isn't nonterminal symbol.");
	}

	initialSymbol = std::move(symbol);
}

} /* namespace grammar */
