/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#include "GraphFromXMLParser.h"
#include "../../sax/ParserException.h"
#include "../../XmlApi.hpp"

#include "../Graph.h"
#include "../GraphClasses.h"

namespace graph {

REPRESENTATION GraphFromXMLParser::parseRepresentation(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "representation");

	REPRESENTATION r = representationFromString(sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER));

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "representation");
	return r;
}

Node GraphFromXMLParser::parseNode(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "node");

	Node n = Node(alib::xmlApi<label::Label>::parse(input));

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "node");
	return n;
}

DirectedEdge GraphFromXMLParser::parseDirectedEdge(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "edge");

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "from");
	Node from = parseNode(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "from");

	label::Label name = alib::xmlApi<label::Label>::parse(input);

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "to");
	Node to = parseNode(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "to");

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "edge");
	return DirectedEdge(std::move(from), std::move(to), std::move(name));
}

UndirectedEdge GraphFromXMLParser::parseUndirectedEdge(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "edge");

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "first");
	Node first = parseNode(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "first");

	label::Label name = alib::xmlApi<label::Label>::parse(input);

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "second");
	Node second = parseNode(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "second");

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "edge");
	return UndirectedEdge(std::move(first), std::move(second), std::move(name));
}

int GraphFromXMLParser::parseValue(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "value");

	int value = std::stoi(sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER));

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "value");
	return value;
}

void GraphFromXMLParser::parseDirectedEdges(std::deque<sax::Token>::iterator& input, DirectedGraph &graph) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "edges");

	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		graph.addEdge(parseDirectedEdge(input));
	}

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "edges");
}

void GraphFromXMLParser::parseUndirectedEdges(std::deque<sax::Token>::iterator& input, UndirectedGraph &graph) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "edges");

	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		graph.addEdge(parseUndirectedEdge(input));
	}

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "edges");
}

void GraphFromXMLParser::parseDirectedEdgeValues(std::deque<sax::Token>::iterator& input, DirectedGraph &graph) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "edgevalues");

	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		graph::DirectedEdge edge = parseDirectedEdge(input);
		int value = parseValue(input);

		graph.setEdgeValue(std::move(edge), value);
	}

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "edgevalues");
}

void GraphFromXMLParser::parseUndirectedEdgeValues(std::deque<sax::Token>::iterator& input, UndirectedGraph &graph) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "edgevalues");

	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		graph::UndirectedEdge edge = parseUndirectedEdge(input);
		int value = parseValue(input);

		graph.setEdgeValue(std::move(edge), value);
	}

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "edgevalues");
}

} // namespace graph

