/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef GRAPH_FROM_XML_PARSER_H_
#define GRAPH_FROM_XML_PARSER_H_

#include "../../sax/FromXMLParserHelper.h"
#include "../GraphFeatures.h"
#include "../../alphabet/SymbolFeatures.h"
#include "Node.h"

namespace graph {

// Parser used to get Graph from XML parsed into list of tokens.
class GraphFromXMLParser {
public:
	static REPRESENTATION parseRepresentation(std::deque<sax::Token>::iterator& input);
	static Node parseNode(std::deque<sax::Token>::iterator& input);
	static DirectedEdge parseDirectedEdge(std::deque<sax::Token>::iterator& input);
	static UndirectedEdge parseUndirectedEdge(std::deque<sax::Token>::iterator& input);
	static int parseValue(std::deque<sax::Token>::iterator& input);

	template<typename T>
	static void parseNodes(std::deque<sax::Token>::iterator& input, T &graph);
	static void parseDirectedEdges(std::deque<sax::Token>::iterator& input, DirectedGraph &graph);
	static void parseUndirectedEdges(std::deque<sax::Token>::iterator& input, UndirectedGraph &graph);

	template<typename T>
	static void parseNodeValues(std::deque<sax::Token>::iterator& input, T &graph);
	static void parseDirectedEdgeValues(std::deque<sax::Token>::iterator& input, DirectedGraph &graph);
	static void parseUndirectedEdgeValues(std::deque<sax::Token>::iterator& input, UndirectedGraph &graph);
};

template<typename T>
void GraphFromXMLParser::parseNodes(std::deque<sax::Token>::iterator& input, T &graph) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "nodes");

	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		graph.addNode(parseNode(input));
	}

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "nodes");
}

template<typename T>
void GraphFromXMLParser::parseNodeValues(std::deque<sax::Token>::iterator& input, T &graph) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "nodevalues");

	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		graph::Node node = parseNode(input);
		int value = parseValue(input);

		graph.setNodeValue(std::move(node), value);
	}

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "nodevalues");
}

} // namespace graph

#endif // GRAPH_FROM_XML_PARSER_H_
