/*
 * TerminatingSymbolAlphabet.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "TerminatingSymbolAlphabet.h"
#include "../../../exception/AlibException.h"

namespace indexes {

TerminatingSymbolAlphabet::TerminatingSymbolAlphabet ( alphabet::Symbol terminatingSymbol ) : terminatingSymbol ( terminatingSymbol ) {
	addSymbol ( std::move ( terminatingSymbol ) );
}

const std::set < alphabet::Symbol > & TerminatingSymbolAlphabet::getAlphabet ( ) const {
	return this->alphabet;
}

void TerminatingSymbolAlphabet::addSymbol ( alphabet::Symbol symbol ) {
	this->alphabet.insert ( std::move ( symbol ) );
}

void TerminatingSymbolAlphabet::setTerminatingSymbol ( alphabet::Symbol terminatingSymbol ) {
	if ( !alphabet.count ( terminatingSymbol ) )
		throw exception::AlibException ( "Symbol " + ( std::string ) terminatingSymbol + " cannot be set as terminating symbol. It is not present in the alphabet." );

	terminatingSymbol = std::move ( terminatingSymbol );
}

const alphabet::Symbol & TerminatingSymbolAlphabet::getTerminatingSymbol ( ) const {
	return terminatingSymbol;
}

} /* namespace indexes */
