/*
 * HexavigesimalLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef HEXAVIGESIMAL_LABEL_H_
#define HEXAVIGESIMAL_LABEL_H_

#include <hexavigesimal>

#include "LabelBase.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class HexavigesimalLabel : public LabelBase {
protected:
	int hexavigesimal;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit HexavigesimalLabel ( int hexavigesimal );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	int getData ( ) const;

	void setData ( int data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const HexavigesimalLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static HexavigesimalLabel parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual LabelBase * next ( ) const;
	virtual LabelBase * inc ( ) &&;
};

} /* namespace label */

namespace std {

template < >
struct compare < label::HexavigesimalLabel > {
	int operator ()( const label::HexavigesimalLabel & first, const label::HexavigesimalLabel & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* HEXAVIGESIMAL_LABEL_H_ */
