/*
 * PrimitiveLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "PrimitiveLabel.h"
#include "../sax/FromXMLParserHelper.h"
#include "Label.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"
#include "UniqueLabel.h"

namespace label {

PrimitiveLabel::PrimitiveLabel(primitive::Primitive primitive) : primitive(std::move(primitive)) {

}

LabelBase* PrimitiveLabel::clone() const {
	return new PrimitiveLabel(*this);
}

LabelBase* PrimitiveLabel::plunder() && {
	return new PrimitiveLabel(std::move(*this));
}

const primitive::Primitive& PrimitiveLabel::getData() const {
	return primitive;
}

primitive::Primitive& PrimitiveLabel::getData() {
	return primitive;
}

int PrimitiveLabel::compare(const PrimitiveLabel& other) const {
	return primitive.getData().compare(other.primitive.getData());
}

void PrimitiveLabel::operator>>(std::ostream& out) const {
	out << "(PrimitiveLabel " << primitive << ")";
}

PrimitiveLabel::operator std::string() const {
	return (std::string) primitive;
}

const std::string PrimitiveLabel::XML_TAG_NAME = "PrimitiveLabel";

PrimitiveLabel PrimitiveLabel::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, PrimitiveLabel::XML_TAG_NAME);
	PrimitiveLabel data(alib::xmlApi<primitive::Primitive>::parse(input));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, PrimitiveLabel::XML_TAG_NAME);
	return data;
}

void PrimitiveLabel::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(PrimitiveLabel::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<primitive::Primitive>::compose(out, primitive);
	out.emplace_back(PrimitiveLabel::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

LabelBase* PrimitiveLabel::next() const {
	return new PrimitiveLabel(primitive.next());
}

LabelBase* PrimitiveLabel::inc() && {
	primitive.inc();
	return NULL;
}

} /* namespace label */

namespace alib {

xmlApi<label::Label>::ParserRegister<label::PrimitiveLabel> primitiveLabelParserRegister = xmlApi<label::Label>::ParserRegister<label::PrimitiveLabel>(label::PrimitiveLabel::XML_TAG_NAME, label::PrimitiveLabel::parse);
xmlApi<alib::Object>::ParserRegister<label::PrimitiveLabel> primitiveLabelParserRegister2 = xmlApi<alib::Object>::ParserRegister<label::PrimitiveLabel>(label::PrimitiveLabel::XML_TAG_NAME, label::PrimitiveLabel::parse);

} /* namespace alib */
