/*
 * RegExp.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "RegExp.h"
#include "../alphabet/Symbol.h"
#include "../string/LinearString.h"
#include "unbounded/UnboundedRegExpSymbol.h"
#include "unbounded/UnboundedRegExpEmpty.h"
#include "unbounded/UnboundedRegExpConcatenation.h"

#include "unbounded/UnboundedRegExp.h"

namespace regexp {

const std::set<alphabet::Symbol>& RegExp::getAlphabet() const {
	return this->getData().getAlphabet();
}

regexp::RegExp regexpFrom( const std::string & string ) {
	regexp::UnboundedRegExpConcatenation con;
	for(const char& symbol : string) {
		con.appendElement( regexp::UnboundedRegExpSymbol( alphabet::symbolFrom( symbol ) ) );
	}
	return regexp::RegExp { regexp::UnboundedRegExp ( std::move ( con ) ) };
}

regexp::RegExp regexpFrom( const char* string ) {
	return regexpFrom( (std::string) string );
}

regexp::RegExp regexpFrom( string::LinearString string ) {
	regexp::UnboundedRegExpConcatenation con;
	for(auto& symbol : string.getContent()) {
		con.appendElement( regexp::UnboundedRegExpSymbol( std::move ( symbol ) ) );
	}
	return regexp::RegExp { regexp::UnboundedRegExp ( std::move( con ) ) };
}

regexp::RegExp regexpFrom( alphabet::Symbol symbol ) {
	return regexp::RegExp { regexp::UnboundedRegExp( regexp::UnboundedRegExpSymbol( std::move ( symbol ) ) ) };
}

regexp::RegExp regexpFrom( ) {
	return regexp::RegExp { regexp::UnboundedRegExp( regexp::UnboundedRegExpEmpty { } ) };
}

} /* namespace regexp */

