/*
 * RegExpToXMLComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_TO_XML_COMPOSER_H_
#define REG_EXP_TO_XML_COMPOSER_H_

#include <deque>
#include "../RegExpFeatures.h"
#include "../unbounded/UnboundedRegExpElement.h"
#include "../formal/FormalRegExpElement.h"
#include "../../sax/Token.h"

namespace regexp {

/**
 * This class contains methods to print XML representation of regular expression to the output stream.
 */
class RegExpToXMLComposer : public UnboundedRegExpElement::const_visitor_type, public FormalRegExpElement::const_visitor_type {
public:
	RegExpToXMLComposer() {}

	static RegExpToXMLComposer REGEXP_TO_XML_COMPOSER;

	static void composeAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& alphabet);

private:
	void Visit(void*, const UnboundedRegExpAlternation& alternation) const;
	void Visit(void*, const UnboundedRegExpConcatenation& concatenation) const;
	void Visit(void*, const UnboundedRegExpIteration& iteration) const;
	void Visit(void*, const UnboundedRegExpSymbol& symbol) const;
	void Visit(void*, const UnboundedRegExpEpsilon& epsilon) const;
	void Visit(void*, const UnboundedRegExpEmpty& empty) const;


	void Visit(void*, const FormalRegExpAlternation& alternation) const;
	void Visit(void*, const FormalRegExpConcatenation& concatenation) const;
	void Visit(void*, const FormalRegExpIteration& iteration) const;
	void Visit(void*, const FormalRegExpSymbol& symbol) const;
	void Visit(void*, const FormalRegExpEpsilon& epsilon) const;
	void Visit(void*, const FormalRegExpEmpty& empty) const;
};

} /* namespace regexp */

#endif /* REG_EXP_TO_XML_COMPOSER_H_ */
