/*
 * FromXMLParserHelper.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "FromXMLParserHelper.h"
#include "ParserException.h"

namespace sax {

bool FromXMLParserHelper::isToken(const std::deque<Token>::const_iterator& input, Token::TokenType type, const std::string& data) {
	return input->getType() == type && input->getData() == data;
}

bool FromXMLParserHelper::isTokenType(const std::deque<Token>::const_iterator& input, Token::TokenType type) {
	return input->getType() == type;
}

void FromXMLParserHelper::popToken(std::deque<Token>::iterator& input, Token::TokenType type, const std::string& data) {
	if (isToken(input, type, data)) {
		++input;
	} else {
		throw ParserException(Token(data, type), *input);
	}
}

std::string FromXMLParserHelper::popTokenData(std::deque<Token>::iterator& input, Token::TokenType type) {
	if(isTokenType(input, type)) {
		std::string result = std::move(*input).moveData();
		++input;
		return result;
	} else {
		throw ParserException(Token("?", type), *input);
	}
}

std::string FromXMLParserHelper::getTokenData(const std::deque<Token>::const_iterator& input, Token::TokenType type) {
	if(isTokenType(input, type)) {
		return input->getData();
	} else {
		throw ParserException(Token("?", type), *input);
	}
}

} /* namespace sax */
