/*
 * Epsilon.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#include "Epsilon.h"

#include "../sax/FromXMLParserHelper.h"
#include "common/StringFromXMLParser.h"
#include "common/StringToXMLComposer.h"
#include "String.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"

namespace string {

Epsilon::Epsilon() {}

Epsilon::Epsilon(std::set<alphabet::Symbol> alphabet) {
	this->alphabet = std::move(alphabet);
}

StringBase* Epsilon::clone() const {
	return new Epsilon(*this);
}

StringBase* Epsilon::plunder() && {
	return new Epsilon(std::move(*this));
}

bool Epsilon::removeSymbolFromAlphabet(const alphabet::Symbol & symbol) {
	return alphabet.erase(symbol);
}

std::vector<alphabet::Symbol> Epsilon::getContent() const {
	return {};
}

bool Epsilon::isEmpty() const {
	return true;
}

int Epsilon::compare(const Epsilon& other) const {
	std::compare<std::set<alphabet::Symbol>> comp;
	return comp(alphabet, other.getAlphabet());
}

void Epsilon::operator>>(std::ostream& out) const {
	out << "(Epsilon)";
}

Epsilon::operator std::string() const {
	return "E";
}

Epsilon Epsilon::EPSILON = Epsilon();

const std::string Epsilon::XML_TAG_NAME = "Epsilon";

Epsilon Epsilon::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Epsilon::XML_TAG_NAME);
	std::set<alphabet::Symbol> alphabet = StringFromXMLParser::parseAlphabet(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Epsilon::XML_TAG_NAME);
	return Epsilon(alphabet);
}

void Epsilon::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(Epsilon::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	StringToXMLComposer::compose(out, alphabet);
	out.emplace_back(Epsilon::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

} /* namespace string */

namespace alib {

xmlApi<string::String>::ParserRegister<string::Epsilon> epsilonParserRegister = xmlApi<string::String>::ParserRegister<string::Epsilon>(string::Epsilon::XML_TAG_NAME, string::Epsilon::parse);
xmlApi<alib::Object>::ParserRegister<string::Epsilon> epsilonParserRegister2 = xmlApi<alib::Object>::ParserRegister<string::Epsilon>(string::Epsilon::XML_TAG_NAME, string::Epsilon::parse);

} /* namespace alib */
