/*
 * TerminatingSymbolStringAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "TerminatingSymbolStringAlphabet.h"
#include "../../exception/AlibException.h"

#include <algorithm>

namespace string {

TerminatingSymbolStringAlphabet::TerminatingSymbolStringAlphabet ( alphabet::Symbol terminatingSymbol ) : terminatingSymbol ( terminatingSymbol ) {
	addSymbolToAlphabet ( std::move ( terminatingSymbol ) );
}

const std::set < alphabet::Symbol > & TerminatingSymbolStringAlphabet::getAlphabet ( ) const {
	return alphabet;
}

bool TerminatingSymbolStringAlphabet::addSymbolToAlphabet ( alphabet::Symbol symbol ) {
	return alphabet.insert ( std::move ( symbol ) ).second;
}

void TerminatingSymbolStringAlphabet::addSymbolsToAlphabet ( const std::set < alphabet::Symbol > & symbols ) {
	for ( const alphabet::Symbol & addedSymbol : symbols )
		addSymbolToAlphabet ( addedSymbol );
}

void TerminatingSymbolStringAlphabet::setAlphabet ( std::set < alphabet::Symbol > newSymbols ) {
	std::set < alphabet::Symbol > removed;
	std::set_difference ( alphabet.begin ( ), alphabet.end ( ), newSymbols.begin ( ), newSymbols.end ( ), std::inserter ( removed, removed.end ( ) ) );

	for ( const alphabet::Symbol & removedSymbol : removed )
		removeSymbolFromAlphabet ( removedSymbol );

	alphabet = std::move ( newSymbols );
}

void TerminatingSymbolStringAlphabet::setTerminatingSymbol ( alphabet::Symbol terminatingSymbol ) {
	if ( !alphabet.count ( terminatingSymbol ) )
		throw exception::AlibException ( "Symbol " + ( std::string ) terminatingSymbol + " cannot be set as terminating symbol. It is not present in the alphabet." );

	terminatingSymbol = std::move ( terminatingSymbol );
}

const alphabet::Symbol & TerminatingSymbolStringAlphabet::getTerminatingSymbol ( ) const {
	return terminatingSymbol;
}

} /* namespace string */
