/*
 * RankedAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "RankedPatternAlphabet.h"
#include "../TreeException.h"

#include <algorithm>

namespace tree {

RankedPatternAlphabet::RankedPatternAlphabet(alphabet::RankedSymbol subtreeWildcard) : subtreeWildcard(subtreeWildcard) {
	if(subtreeWildcard.getRank().getData() != 0)
		throw TreeException("Subtree wildcard symbol " + (std::string) subtreeWildcard + " has nonzero rank");
	alphabet.insert(std::move(subtreeWildcard));
}

void RankedPatternAlphabet::setSubtreeWildcard(alphabet::RankedSymbol symbol) {
	if (!alphabet.count(symbol))
		throw TreeException("Subtree wildcard symbol \"" + (std::string) symbol + "\" is not in the alphabet.");

	if(subtreeWildcard.getRank().getData() != 0)
		throw TreeException("Subtree wildcard symbol " + (std::string) symbol + " has nonzero rank");

	subtreeWildcard = std::move(symbol);

}

const alphabet::RankedSymbol& RankedPatternAlphabet::getSubtreeWildcard() const {
	return subtreeWildcard;
}

} /* namespace tree */

