/*
 * PrefixRankedBarPattern.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef PREFIX_RANKED_BAR_PATTERN_H_
#define PREFIX_RANKED_BAR_PATTERN_H_

#include <set>
#include <vector>

#include "../../alphabet/RankedSymbol.h"
#include "../TreeBase.h"
#include "../common/RankedPatternAlphabet.h"
#include "RankedNode.h"

namespace tree {

class RankedPattern;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of LinearStringElement.
 */
class PrefixRankedBarPattern : public TreeBase, public RankedPatternAlphabet {
	std::vector < alphabet::RankedSymbol > m_Data;

	alphabet::Symbol bar;
	alphabet::Symbol variablesBar;

public:
	explicit PrefixRankedBarPattern ( alphabet::Symbol bar, alphabet::Symbol variablesBar, alphabet::RankedSymbol subtreeWildcard, std::set < alphabet::RankedSymbol > alphabet, std::vector < alphabet::RankedSymbol > data );
	explicit PrefixRankedBarPattern ( alphabet::Symbol bar, alphabet::Symbol variablesBar, alphabet::RankedSymbol subtreeWildcard, std::vector < alphabet::RankedSymbol > data );
	explicit PrefixRankedBarPattern ( alphabet::Symbol bar, alphabet::Symbol variablesBar, const RankedPattern & tree );

	void toPrefixRankedBar ( const RankedNode & node );

	virtual TreeBase * clone ( ) const;
	virtual TreeBase * plunder ( ) &&;

	virtual bool removeSymbolFromAlphabet ( const alphabet::RankedSymbol & symbol );

	/**
	 * @return List of symbols forming tree (const version).
	 */
	const std::vector < alphabet::RankedSymbol > & getContent ( ) const;

	void setContent ( std::vector < alphabet::RankedSymbol > data );

	const alphabet::Symbol & getBarSymbol ( ) const;

	const alphabet::Symbol & getVariablesBarSymbol ( ) const;

	void arityChecksum ( const std::vector < alphabet::RankedSymbol > & data );

	/**
	 * @return true if tree is an empty word (vector length is 0)
	 */
	bool isEmpty ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const PrefixRankedBarPattern & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static PrefixRankedBarPattern parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace tree */

namespace std {

template < >
struct compare < ::tree::PrefixRankedBarPattern > {
	int operator ()( const::tree::PrefixRankedBarPattern & first, const::tree::PrefixRankedBarPattern & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* PREFIX_RANKED_BAR_PATTERN_H_ */
