/*
 * UnrankedNode.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Stepan Plachy
 */

#ifndef UNRANKED_NODE_H_
#define UNRANKED_NODE_H_

#include "../../alphabet/Symbol.h"
#include "../../alphabet/RankedSymbol.h"
#include <vector>
#include <set>
#include "../common/UnrankedAlphabet.h"

namespace tree {

class RankedNode;

/**
 * Represents a node in the ranked tree. Contains name of the symbol.
 */
class UnrankedNode {
protected:
	alphabet::Symbol symbol;

	std::vector<UnrankedNode*> children;

	UnrankedNode* parent;

	/**
	 * Parent tree contanining this instance of RegExpElement
	 */
	const UnrankedAlphabet * parentTree;

	/**
	 * @copydoc UnrankedNode::cloneAsUnranked() const
	 */
	RankedNode* cloneAsRanked() const;

	/**
	 * @copydoc UnrankedNode::testSymbol() const
	 */
	bool testSymbol( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc UnrankedNode::attachTree()
	 */
	bool attachTree ( const UnrankedAlphabet * tree );

	/**
	 * @copydoc UnrankedNode::computeMinimalAlphabet()
	 */
	void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;
public:
	explicit UnrankedNode(alphabet::Symbol symbol, std::vector<UnrankedNode *> children);

	UnrankedNode(const UnrankedNode& other);
	UnrankedNode(UnrankedNode&& other) noexcept;
	UnrankedNode& operator=(const UnrankedNode& other);
	UnrankedNode& operator=(UnrankedNode&& other) noexcept;
	~UnrankedNode() noexcept;

	/**
	 * @copydoc UnrankedNode::clone() const
	 */
	UnrankedNode* clone() const;

	/**
	 * @copydoc UnrankedNode::plunder() const
	 */
	UnrankedNode* plunder() &&;

	/**
	 * @return children
	 */
	const std::vector<const UnrankedNode*> & getChildren() const;

	/**
	 * @return children
	 */
	const std::vector<UnrankedNode*> & getChildren();

	UnrankedNode * getParent();

	const UnrankedNode * getParent() const;

	void setSymbol(alphabet::Symbol symbol);

	void pushBackChild(const UnrankedNode & node);

	void swap(UnrankedNode& other);

	int compare(const UnrankedNode&) const;

	/**
	 * @copydoc UnrankedNode::operator>>() const
	 */
	void operator>>(std::ostream& out) const;

	friend std::ostream& operator<<(std::ostream&, const UnrankedNode& node);

	/**
	 * Returns the string representation of Node's Symbol.
	 */
	const alphabet::Symbol& getSymbol() const;

	explicit operator std::string() const;

	void nicePrint(std::ostream &, const std::string & = "", const bool = true) const;

	friend class UnrankedTree;
	friend class UnrankedPattern;
	friend class RankedTree;
	friend class RankedPattern;
};

} /* namespace tree */

#endif /* UNRANKED_NODE_H_ */

