#include "DispatchTest.h"
#include "common/multipleDispatch.hpp"
#include "common/base.hpp"
#include <set>
#include <string>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( DispatchTest, "std" );
CPPUNIT_TEST_SUITE_REGISTRATION ( DispatchTest );

void DispatchTest::setUp ( ) {
}

void DispatchTest::tearDown ( ) {
}

namespace dispatch {

class TmpBase : public alib::base < TmpBase > {
};

class Tmp1 : public TmpBase {
	int data;

public:
	Tmp1 ( int data ) : data ( data ) {
	}

	TmpBase * clone ( ) const {
		return new Tmp1 ( * this );
	}

	TmpBase * plunder ( ) && {
		return new Tmp1 ( * this );
	}

	virtual int compare ( const TmpBase & other ) const {
		return -other.compare ( * this );
	}

	virtual int compare ( const Tmp1 & other ) const {
		return this->data - other.data;
	}

	virtual void operator >>( std::ostream & os ) const {
		os << "Tmp1(" << data << ")";
	}

	virtual operator std::string ( ) const {
		return "Tmp1(" + std::to_string ( data ) + ")";
	}

	int getData ( ) const {
		return data;
	}

};

class Tmp2 : public TmpBase {
	double data;

public:
	Tmp2 ( double data ) : data ( data ) {
	}

	Tmp2 ( const Tmp1 & other ) : data ( other.getData ( ) ) {
	}

	TmpBase * clone ( ) const {
		return new Tmp2 ( * this );
	}

	TmpBase * plunder ( ) && {
		return new Tmp2 ( * this );
	}

	virtual int compare ( const TmpBase & other ) const {
		return -other.compare ( * this );
	}

	virtual int compare ( const Tmp2 & other ) const {
		return this->data - other.data;
	}

	virtual void operator >>( std::ostream & os ) const {
		os << "Tmp2(" << data << ")";
	}

	virtual operator std::string ( ) const {
		return "Tmp2(" + std::to_string ( data ) + ")";
	}

	double getData ( ) const {
		return data;
	}

};

class Tmp3 : public TmpBase {
	std::string data;

public:
	Tmp3 ( const std::string & data ) : data ( data ) {
	}

	Tmp3 ( const Tmp1 & other ) : data ( std::to_string ( other.getData ( ) ) ) {
	}

	Tmp3 ( const Tmp2 & other ) : data ( std::to_string ( other.getData ( ) ) ) {
	}

	TmpBase * clone ( ) const {
		return new Tmp3 ( * this );
	}

	TmpBase * plunder ( ) && {
		return new Tmp3 ( * this );
	}

	virtual int compare ( const TmpBase & other ) const {
		return -other.compare ( * this );
	}

	virtual int compare ( const Tmp3 & other ) const {
		return this->data.compare ( other.data );
	}

	virtual void operator >>( std::ostream & os ) const {
		os << "Tmp3(" << data << ")";
	}

	virtual operator std::string ( ) const {
		return "Tmp3(" + data + ")";
	}

	const std::string & getData ( ) const {
		return data;
	}

};

// -------------------------------------------------------------------------------------------------------------------------------------------------------

class TmpVisitor : public std::SingleDispatch < int, TmpBase > {
public:
	static int eval ( const Tmp2 & first ) {
		std::cout << first << std::endl;

		return 2;
	}

	static int eval ( const Tmp3 & first ) {
		std::cout << first << std::endl;

		return 3;
	}

	static int eval ( const TmpBase & first ) {
		return getInstance ( ).dispatch ( first );
	}

	static TmpVisitor & getInstance ( ) {
		static TmpVisitor res;

		return res;
	}

};

int TmpVisitorExtensionTmp1 ( const Tmp1 & first ) {
	std::cout << first << std::endl;

	return 1;
}

TmpVisitor::RegistratorWrapper < int, Tmp1 > TmpVisitorTmp1 = TmpVisitor::RegistratorWrapper < int, Tmp1 > ( TmpVisitor::getInstance ( ), TmpVisitorExtensionTmp1 );
TmpVisitor::RegistratorWrapper < int, Tmp2 > TmpVisitorTmp2 = TmpVisitor::RegistratorWrapper < int, Tmp2 > ( TmpVisitor::getInstance ( ), TmpVisitor::eval );
TmpVisitor::RegistratorWrapper < int, Tmp3 > TmpVisitorTmp3 = TmpVisitor::RegistratorWrapper < int, Tmp3 > ( TmpVisitor::getInstance ( ), TmpVisitor::eval );

} /* namespace dispatch */

void DispatchTest::testDispatch ( ) {
	dispatch::Tmp1 tmp1 ( 2 );

	int a = dispatch::TmpVisitor::eval ( tmp1 );

	CPPUNIT_ASSERT ( a == 1 );

	a = dispatch::TmpVisitor::eval ( ( dispatch::TmpBase & ) tmp1 );
	CPPUNIT_ASSERT ( a == 1 );

	dispatch::Tmp2 tmp2 ( 3.3 );

	a = dispatch::TmpVisitor::eval ( tmp2 );
	CPPUNIT_ASSERT ( a == 2 );

	a = dispatch::TmpVisitor::eval ( ( dispatch::TmpBase & ) tmp2 );
	CPPUNIT_ASSERT ( a == 2 );
}

namespace dispatch {

// -------------------------------------------------------------------------------------------------------------------------------------------------------

class TmpVisitor2 : public std::SingleDispatchFirstStaticParam < void, int &, TmpBase > {
public:
	static void eval ( int & res, const Tmp2 & first ) {
		std::cout << first << std::endl;

		res = 2;
	}

	static void eval ( int & res, const Tmp3 & first ) {
		std::cout << first << std::endl;

		res = 3;
	}

	static void eval ( int & res, const TmpBase & first ) {
		getInstance ( ).dispatch ( res, first );
	}

	static TmpVisitor2 & getInstance ( ) {
		static TmpVisitor2 res;

		return res;
	}

};

void TmpVisitor2ExtensionTmp1 ( int & res, const Tmp1 & first ) {
	std::cout << first << std::endl;

	res = 1;
}

TmpVisitor2::RegistratorWrapper < void, Tmp1 > TmpVisitor2Tmp1 = TmpVisitor2::RegistratorWrapper < void, Tmp1 > ( TmpVisitor2::getInstance ( ), TmpVisitor2ExtensionTmp1 );
TmpVisitor2::RegistratorWrapper < void, Tmp2 > TmpVisitor2Tmp2 = TmpVisitor2::RegistratorWrapper < void, Tmp2 > ( TmpVisitor2::getInstance ( ), TmpVisitor2::eval );
TmpVisitor2::RegistratorWrapper < void, Tmp3 > TmpVisitor2Tmp3 = TmpVisitor2::RegistratorWrapper < void, Tmp3 > ( TmpVisitor2::getInstance ( ), TmpVisitor2::eval );

} /* namespace dispatch */

void DispatchTest::testDispatch2 ( ) {
	dispatch::Tmp1 tmp1 ( 2 );

	int a;
	dispatch::TmpVisitor2::eval ( a, tmp1 );

	CPPUNIT_ASSERT ( a == 1 );

	dispatch::TmpVisitor2::eval ( a, ( dispatch::TmpBase & ) tmp1 );
	CPPUNIT_ASSERT ( a == 1 );

	dispatch::Tmp2 tmp2 ( 3.3 );

	dispatch::TmpVisitor2::eval ( a, tmp2 );
	CPPUNIT_ASSERT ( a == 2 );

	dispatch::TmpVisitor2::eval ( a, ( dispatch::TmpBase & ) tmp2 );
	CPPUNIT_ASSERT ( a == 2 );
}
