/*
 * AllEpsilonClosure.h
 *
 *  Created on: 12. 4. 2015
 *	  Author: Jan Travnicek
 */

#ifndef EFFICIENT_ALL_EPSILON_CLOSURE_H_
#define EFFICIENT_ALL_EPSILON_CLOSURE_H_

#include <common/multipleDispatch.hpp>
#include <set>
#include <map>

#include "automaton/Automaton.h"
#include <automaton/common/State.h>

namespace automaton {

namespace properties {

namespace efficient {

class AllEpsilonClosure : public std::SingleDispatch<std::map<automaton::State, std::set<automaton::State>>, automaton::AutomatonBase> {
public:
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::Automaton & automaton);

	/**
	 * Computes allEpsilon closure of a state in allEpsilon nonfree automaton
	 */
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::EpsilonNFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::MultiInitialStateNFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::NFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::DFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::ExtendedNFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::CompactNFA & fsm);

	static AllEpsilonClosure& getInstance() {
		static AllEpsilonClosure res;
		return res;
	}
};

} /* namespace efficient */

} /* namespace properties */

} /* namespace automaton */

#endif /* EFFICIENT_ALL_EPSILON_CLOSURE_H_ */
