/*
 * EpsilonRemoverIncoming.h
 *
 *  Created on: 24. 11. 2014
 *	  Author: Jan Travnicek
 */

#ifndef EFFICIENT_EPSILON_REMOVER_INCOMING_H_
#define EFFICIENT_EPSILON_REMOVER_INCOMING_H_

#include <common/multipleDispatch.hpp>
#include <map>
#include <algorithm>

#include <automaton/Automaton.h>

#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>
#include <exception/AlibException.h>

namespace automaton {

namespace simplify {

namespace efficient {

class EpsilonRemoverIncoming : public std::SingleDispatch<automaton::Automaton, automaton::AutomatonBase> {
public:
	static automaton::Automaton remove( const automaton::Automaton & automaton );

	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	static automaton::NFA remove( const automaton::EpsilonNFA & fsm );
	static automaton::MultiInitialStateNFA remove( const automaton::MultiInitialStateNFA & fsm );
	static automaton::NFA remove( const automaton::NFA & fsm );
	static automaton::DFA remove( const automaton::DFA & fsm );

	static EpsilonRemoverIncoming& getInstance() {
		static EpsilonRemoverIncoming res;
		return res;
	}
};

} /* namespace efficient */

} /* namespace simplify */

} /* namespace automaton */

#endif /* EFFICIENT_EPSILON_REMOVER_INCOMING_H_ */
