/*
 * clone.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef CHRONO_HPP_
#define CHRONO_HPP_

namespace std {

namespace chrono {

class measure_duration {
	time_point<system_clock> m_Start;
	time_point<system_clock> m_End;

public:
	void start() {
		m_Start = system_clock::now();
	}

	void end() {
		m_End = system_clock::now();
	}

	microseconds duration() {
		return duration_cast<microseconds>(m_End - m_Start);
	}

};

class measurements {
public:
	enum Type {
		OVERALL = 1,
		INIT = 2,
		FINALIZE = 4,
		MAIN = 8,
		AUXILARY = 16
	};

	struct Format {
		unsigned type;
	
		static Format LIST;
		static Format TREE;
	};

	struct Filter {
		bool set;
		unsigned val;

		static Filter ALL;
		static Filter NONE;
		static Filter OVERALL;
		static Filter NO_OVERALL;
		static Filter INIT;
		static Filter NO_INIT;
		static Filter FINALIZE;
		static Filter NO_FINALIZE;
		static Filter MAIN;
		static Filter NO_MAIN;
		static Filter AUXILARY;
		static Filter NO_AUXILARY;

	};

	struct Results {
		const std::vector<std::tuple<std::string, measurements::Type, microseconds, microseconds, std::set<unsigned>>>& results;

		const std::tuple<std::string, measurements::Type, microseconds, microseconds, std::set<unsigned>>& operator[](int idx) const {
			return results[idx];
		}
	};

	static const int measurements_format_index;
	static const int measurements_filter_index;

private:
	std::vector<std::tuple<std::string, measurements::Type, microseconds, microseconds, std::set<unsigned>>> measurements_vector; // name, type, complete duration, real duration
	std::deque<std::tuple<std::string, measurements::Type, time_point<system_clock>, microseconds, std::set<unsigned>>> measurements_stack; // name, type, start, sub measurements duration

	static measurements INSTANCE;

public:
	static void start(std::string name, measurements::Type type) {
		INSTANCE.measurements_stack.emplace_back(std::move(name), type, system_clock::now(), microseconds(0), std::set<unsigned>{});
	}

	static void end() {
		if(INSTANCE.measurements_stack.empty()) throw std::logic_error("no measurement started");

		std::tuple<std::string, measurements::Type, time_point<system_clock>, microseconds, std::set<unsigned>> current = std::move(INSTANCE.measurements_stack.back());
		INSTANCE.measurements_stack.pop_back();

		microseconds dur = duration_cast<microseconds>(system_clock::now() - std::get<2>(current));

		unsigned index = INSTANCE.measurements_vector.size();
		INSTANCE.measurements_vector.emplace_back(std::move(std::get<0>(current)), std::get<1>(current), dur, dur - std::get<3>(current), std::get<4>(current));
		if(!INSTANCE.measurements_stack.empty()) {
			std::get<3>(INSTANCE.measurements_stack.back()) += std::get<2>(INSTANCE.measurements_vector.back());
			std::get<4>(INSTANCE.measurements_stack.back()).insert(index);
		}
	}

	static Results results() {
		return { INSTANCE.measurements_vector };
	}

};

std::ostream& operator<<(std::ostream& out, const measurements::Results& data);

std::ostream& operator<<(std::ostream& out, microseconds time);

std::ostream& operator<<(std::ostream& out, measurements::Type type);

template<typename _CharT, typename _Traits>
inline basic_ostream<_CharT, _Traits>& operator <<( basic_ostream<_CharT, _Traits>& x, measurements::Format f) {
	x.iword(measurements::measurements_format_index) = f.type;
	return x;
}

template<typename _CharT, typename _Traits>
inline basic_ostream<_CharT, _Traits>& operator <<( basic_ostream<_CharT, _Traits>& x, measurements::Filter f) {
	if(f.set)
		x.iword(measurements::measurements_filter_index) &= ~f.val;
	else
		x.iword(measurements::measurements_filter_index) |= f.val;
	return x;
}

} /* namespace chrono */

} /* namespace std */

#endif /* CHRONO_HPP_ */
