/*
 * AutomatonToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_TO_STRING_COMPOSER_H_
#define AUTOMATON_TO_STRING_COMPOSER_H_

#include <ostream>
#include <common/multipleDispatch.hpp>
#include "automaton/Automaton.h"
#include "automaton/AutomatonFeatures.h"

namespace automaton {

class AutomatonToStringComposer : public std::SingleDispatchFirstStaticParam<void, std::ostream&, AutomatonBase> {
	static void composeTransitionsFromState(std::ostream& out, const DFA& automaton, const State& from);
	static void composeTransitionsFromState(std::ostream& out, const NFA& automaton, const State& from);
	static void composeTransitionsFromState(std::ostream& out, const MultiInitialStateNFA& automaton, const State& from);
	static void composeTransitionsFromState(std::ostream& out, const EpsilonNFA& automaton, const State& from);

public:
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	static void compose(std::ostream& output, const Automaton& automaton);

	static void compose(std::ostream& output, const DFA& automaton);
	static void compose(std::ostream& output, const NFA& automaton);
	static void compose(std::ostream& output, const MultiInitialStateNFA& automaton);
	static void compose(std::ostream& output, const EpsilonNFA& automaton);

	static AutomatonToStringComposer& getInstance() {
		static AutomatonToStringComposer res;
		return res;
	}
};

} /* namespace automaton */

#endif /* AUTOMATON_TO_STRING_COMPOSER_H_ */
