/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#include "GraphFromStringLexer.h"

namespace graph {

GraphFromStringLexer::Token GraphFromStringLexer::next(std::istream &in) const
{
	Token token;
	token.type = TokenType::ERROR;
	char character;

L0:
	character = in.get();

	if (in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if (isspace(character)) {
		token.raw += character;
		goto L0;
	} else if (character == '(') {
		token.type = TokenType::LPAR;
		token.value += character;
		token.raw += character;
		return token;
	} else if (character == ')') {
		token.type = TokenType::RPAR;
		token.value += character;
		token.raw += character;
		return token;
	} else if (character == ':') {
		token.type = TokenType::COLON;
		token.value += character;
		token.raw += character;
		return token;
	} else if (character == ',') {
		token.type = TokenType::COMMA;
		token.value += character;
		token.raw += character;
		return token;
	} else if (character == '=') {
		token.type = TokenType::EQUAL;
		token.value += character;
		token.raw += character;
		return token;
	} else if (character >= '0' && character <= '9') {
		token.type = TokenType::INTEGER;
		token.value += character;
		token.raw += character;
		goto L1;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}

L1:
	character = in.get();
	if (in.eof()) {
		return token;
	} else if (character >= '0' && character <= '9') {
		token.value += character;
		token.raw += character;
		goto L1;
	} else {
		in.unget();
		return token;
	}
}

GraphFromStringLexer::Token GraphFromStringLexer::peek(std::istream &in) const
{
	Token token = next(in);
	putback(in, token);
	return token;
}

void GraphFromStringLexer::putback(std::istream &input, GraphFromStringLexer::Token token) const
{
	while (!token.raw.empty()) {
		input.putback(token.raw.back());
		token.raw.pop_back();
	}
	input.clear();
}

std::string GraphFromStringLexer::getString(std::istream &input) const
{
	std::string str;
	char character;

	while (input >> character) {
		if (character == ':') {
			input.putback(character);
			return str;
		}
		str += character;
	}

	return std::string();
}

} // namespace graph

