/*
 * araw.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
**/

#include <tclap/CmdLine.h>
#include <string>
#include <exception/AlibException.h>
#include <RawApi.hpp>
#include <factory/XmlDataFactory.hpp>
#include <sax/SaxParseInterface.h>
#include <sax/ParserException.h>
#include <object/Object.h>

int main(int argc, char** argv) {
	try {
		TCLAP::CmdLine cmd("raw reading binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to read",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg tree_from_raw(		"", "tree_from_raw",	"Convert tree from raw representation (xml)");
		cmd.add( tree_from_raw );

		TCLAP::SwitchArg tree_to_raw(		"", "tree_to_raw",	"Convert tree to raw representation (xml)");
		cmd.add( tree_to_raw );

		TCLAP::SwitchArg string_from_raw(	"", "string_from_raw",	"Convert string from raw representation (string)");
		cmd.add( string_from_raw );

		TCLAP::SwitchArg string_to_raw(		"", "string_to_raw",	"Convert string to raw representation (string)");
		cmd.add( string_to_raw );

		TCLAP::SwitchArg matrix_from_raw(	"", "matrix_from_raw",	"Convert string from raw representation (csv)");
		cmd.add( matrix_from_raw );

		TCLAP::SwitchArg matrix_to_raw(		"", "matrix_to_raw",	"Convert string to raw representation (csv)");
		cmd.add( matrix_to_raw );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		std::chrono::measurements::start("Overal", std::chrono::measurements::Type::OVERALL);
		std::chrono::measurements::start("Input read", std::chrono::measurements::Type::AUXILARY);

		if(tree_from_raw.getValue()) {
			std::deque<sax::Token> tokens;
			if(input.isSet()) {
				if(input.getValue() == "-") {
					sax::SaxParseInterface::parseStdin(tokens);
				} else {
					sax::SaxParseInterface::parseFile(input.getValue(), tokens);
				}
			} else {
				sax::SaxParseInterface::parseStdin(tokens);
			}

			std::deque<sax::Token>::iterator iter = tokens.begin();
			tree::Tree tree = alib::FromRawParsers::treeParser.parseTree(iter);

			std::chrono::measurements::end();
			std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

			alib::XmlDataFactory::toStdout(tree);
		} else if(tree_to_raw.getValue()) {
			std::deque<sax::Token> tokens;
			if(input.isSet()) {
				if(input.getValue() == "-") {
					sax::SaxParseInterface::parseStdin(tokens);
				} else {
					sax::SaxParseInterface::parseFile(input.getValue(), tokens);
				}
			} else {
				sax::SaxParseInterface::parseStdin(tokens);
			}

			tree::Tree tree = alib::XmlDataFactory::fromTokens<tree::Tree>(tokens);

			std::chrono::measurements::end();
			std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

			std::deque<sax::Token> tokens2;
			tree::TreeToRawComposer::compose(tokens2, tree);
			sax::SaxComposeInterface::printStdout(tokens2);
		} else {
			throw exception::AlibException("Invalid input format specified");
		}

		std::chrono::measurements::end();
		std::chrono::measurements::end();

		if(measure.getValue()) std::clog << std::chrono::measurements::results() << std::endl;

		return 0;
	} catch(const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
