/*
 * AutomataStat.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "AutomataStat.h"

#include <exception/AlibException.h>

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <factory/XmlDataFactory.hpp>
#include <container/ObjectsSet.h>
#include <container/ObjectsMap.h>
#include <iostream>

void AutomataStat::stat(const automaton::Automaton& automaton, const AutomataSettings& settings) {
	getInstance().dispatch(automaton.getData(), settings);
}

void AutomataStat::stat(const automaton::NFA& automaton, const AutomataSettings& settings) {
	switch(settings.states) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getStates());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getStates().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getStates().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getStates());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.finalStates) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getFinalStates());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getFinalStates().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getFinalStates().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getFinalStates());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.initialStates) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getInitialState());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << 1 << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << 1 << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getInitialState());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.inputAlphabet) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getInputAlphabet());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getInputAlphabet().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getInputAlphabet().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getInputAlphabet());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.transitions) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getTransitions());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getTransitions().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getTransitions().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getTransitions());
		break;
	case PrintingOptions::NOOP:
		break;
	}
}

auto AutomataStatNFA = AutomataStat::RegistratorWrapper<void, automaton::NFA>(AutomataStat::getInstance(), AutomataStat::stat);

void AutomataStat::stat(const automaton::DFA& automaton, const AutomataSettings& settings) {
	switch(settings.states) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getStates());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getStates().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getStates().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getStates());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.finalStates) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getFinalStates());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getFinalStates().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getFinalStates().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getFinalStates());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.initialStates) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getInitialState());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << 1 << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << 1 << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getInitialState());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.inputAlphabet) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getInputAlphabet());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getInputAlphabet().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getInputAlphabet().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getInputAlphabet());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.transitions) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getTransitions());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getTransitions().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getTransitions().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getTransitions());
		break;
	case PrintingOptions::NOOP:
		break;
	}
}

auto AutomataStatDFA = AutomataStat::RegistratorWrapper<void, automaton::DFA>(AutomataStat::getInstance(), AutomataStat::stat);

